# Generated by ariadne-codegen
# Source: queries.graphql

from typing import Literal, Optional, Union

from pydantic import Field

from .base_model import BaseModel
from .fragments import InvalidInputSelection, OperationErrorSelection, UserSelection


class UsersSetRole(BaseModel):
    admin: "UsersSetRoleAdmin"


class UsersSetRoleAdmin(BaseModel):
    user: "UsersSetRoleAdminUser"


class UsersSetRoleAdminUser(BaseModel):
    mutate: Optional["UsersSetRoleAdminUserMutate"]


class UsersSetRoleAdminUserMutate(BaseModel):
    set_role: Union[
        "UsersSetRoleAdminUserMutateSetRoleUser",
        "UsersSetRoleAdminUserMutateSetRoleInvalidInput",
        "UsersSetRoleAdminUserMutateSetRoleOperationError",
    ] = Field(alias="setRole", discriminator="typename__")


class UsersSetRoleAdminUserMutateSetRoleUser(UserSelection):
    typename__: Literal["User"] = Field(alias="__typename")


class UsersSetRoleAdminUserMutateSetRoleInvalidInput(InvalidInputSelection):
    typename__: Literal["InvalidInput"] = Field(alias="__typename")


class UsersSetRoleAdminUserMutateSetRoleOperationError(OperationErrorSelection):
    typename__: Literal["OperationError"] = Field(alias="__typename")


UsersSetRole.model_rebuild()
UsersSetRoleAdmin.model_rebuild()
UsersSetRoleAdminUser.model_rebuild()
UsersSetRoleAdminUserMutate.model_rebuild()



from typing import List

from . import types, util


def list() -> List[types.Role]:
    """
    Get a list of all roles in the registry.
    """
    return [types.role_adapter.validate_python(r.model_dump()) for r in util.get_client().roles_list()]



import typing as T

import boto3


class BuildClientBase(T.Protocol):
    def __call__(self, session: boto3.Session, client_kwargs: dict[str, T.Any], **kwargs): ...


class BuildClientHook(T.Protocol):
    def __call__(
        self, build_client_base: BuildClientBase, session: boto3.Session, client_kwargs: dict[str, T.Any], **kwargs
    ): ...


_build_client_hook = None


def get_build_s3_client_hook() -> T.Optional[BuildClientHook]:
    """
    Return build S3 client hook.
    """

    return _build_client_hook


def set_build_s3_client_hook(hook: T.Optional[BuildClientHook]) -> T.Optional[BuildClientHook]:
    """
    Set build S3 client hook.

    Example for overriding `ServerSideEncryption` parameter for certain S3 operations:

    ```python
    from quilt3.hooks import set_build_s3_client_hook

    def event_handler(params, **kwargs):
        # Be mindful with parameters you set here.
        # Specifically it's not recommended to override/delete already set parameters
        # because that can break quilt3 logic.
        params.setdefault("ServerSideEncryption", "AES256")

    def hook(build_client_base, session, client_kwargs, **kwargs):
        client = build_client_base(session, client_kwargs, **kwargs)
        # Docs for boto3 events system we use below:
        # https://boto3.amazonaws.com/v1/documentation/api/latest/guide/events.html
        for op in (
            "CreateMultipartUpload",
            "CopyObject",
            "PutObject",
        ):
            client.meta.events.register(f"before-parameter-build.s3.{op}", event_handler)
        return client

    old_hook = set_build_s3_client_hook(hook)
    ```

    Args:
        hook: Build client hook.

    Returns:
        Old build client hook.
    """
    global _build_client_hook
    old_hook = _build_client_hook
    _build_client_hook = hook
    return old_hook



# Generated by ariadne-codegen
# Source: queries.graphql

from typing import Literal, Union

from pydantic import Field

from .base_model import BaseModel
from .fragments import InvalidInputSelection, OperationErrorSelection


class BucketTabulatorTableSet(BaseModel):
    admin: "BucketTabulatorTableSetAdmin"


class BucketTabulatorTableSetAdmin(BaseModel):
    bucket_set_tabulator_table: Union[
        "BucketTabulatorTableSetAdminBucketSetTabulatorTableBucketConfig",
        "BucketTabulatorTableSetAdminBucketSetTabulatorTableInvalidInput",
        "BucketTabulatorTableSetAdminBucketSetTabulatorTableOperationError",
    ] = Field(alias="bucketSetTabulatorTable", discriminator="typename__")


class BucketTabulatorTableSetAdminBucketSetTabulatorTableBucketConfig(BaseModel):
    typename__: Literal["BucketConfig"] = Field(alias="__typename")


class BucketTabulatorTableSetAdminBucketSetTabulatorTableInvalidInput(InvalidInputSelection):
    typename__: Literal["InvalidInput"] = Field(alias="__typename")


class BucketTabulatorTableSetAdminBucketSetTabulatorTableOperationError(OperationErrorSelection):
    typename__: Literal["OperationError"] = Field(alias="__typename")


BucketTabulatorTableSet.model_rebuild()
BucketTabulatorTableSetAdmin.model_rebuild()



"""
search_util.py

Contains search-related glue code
"""

import json
import typing as T

from . import session


def search_api(query: T.Union[str, dict], index: str, limit: int = 10):
    """
    Send a query to the search API
    """
    if isinstance(query, dict):
        params = dict(index=index, action="freeform", body=json.dumps(query), size=limit)
    else:
        params = dict(index=index, action="search", query=query, size=limit)
    response = session.get_session().get(
        f"{session.get_registry_url()}/api/search",
        params=params,
    )
    return response.json()



import typing as T

from . import exceptions, types, util


def list_tables(bucket_name: str) -> list[types.TabulatorTable]:
    """
    List all tabulator tables in a bucket.
    """
    result = util.get_client().bucket_tabulator_tables_list(bucket_name)
    if result is None:
        raise exceptions.BucketNotFoundError
    return [types.TabulatorTable(**x.model_dump()) for x in result.tabulator_tables]


def set_table(bucket_name: str, table_name: str, config: T.Optional[str]) -> None:
    """
    Set the tabulator table configuration. Pass `None` to remove the table.
    """
    result = util.get_client().bucket_tabulator_table_set(bucket_name, table_name, config)
    util.handle_errors(result)


def rename_table(bucket_name: str, table_name: str, new_table_name: str) -> None:
    """
    Rename tabulator table.
    """
    result = util.get_client().bucket_tabulator_table_rename(bucket_name, table_name, new_table_name)
    util.handle_errors(result)


def get_open_query() -> bool:
    """
    Get the **open query** status.
    """
    return util.get_client().tabulator_get_open_query()


def set_open_query(enabled: bool) -> None:
    """
    Set the **open query** status.
    """
    util.get_client().tabulator_set_open_query(enabled)



# Generated by ariadne-codegen
# Source: queries.graphql

from typing import Optional

from .base_model import BaseModel
from .fragments import UserSelection


class UsersGet(BaseModel):
    admin: "UsersGetAdmin"


class UsersGetAdmin(BaseModel):
    user: "UsersGetAdminUser"


class UsersGetAdminUser(BaseModel):
    get: Optional["UsersGetAdminUserGet"]


class UsersGetAdminUserGet(UserSelection):
    pass


UsersGet.model_rebuild()
UsersGetAdmin.model_rebuild()
UsersGetAdminUser.model_rebuild()



from . import _graphql_client, exceptions, types


def handle_errors(result: _graphql_client.BaseModel) -> _graphql_client.BaseModel:
    if isinstance(result, (_graphql_client.InvalidInputSelection, _graphql_client.OperationErrorSelection)):
        raise exceptions.Quilt3AdminError(result)
    return result


def handle_user_mutation(result: _graphql_client.BaseModel) -> types.User:
    return types.User(**handle_errors(result).model_dump())


def get_client():
    return _graphql_client.Client()



# Generated by ariadne-codegen
# Source: queries.graphql

from typing import Literal, Optional, Union

from pydantic import Field

from .base_model import BaseModel
from .fragments import InvalidInputSelection, OperationErrorSelection, UserSelection


class UsersRemoveRoles(BaseModel):
    admin: "UsersRemoveRolesAdmin"


class UsersRemoveRolesAdmin(BaseModel):
    user: "UsersRemoveRolesAdminUser"


class UsersRemoveRolesAdminUser(BaseModel):
    mutate: Optional["UsersRemoveRolesAdminUserMutate"]


class UsersRemoveRolesAdminUserMutate(BaseModel):
    remove_roles: Union[
        "UsersRemoveRolesAdminUserMutateRemoveRolesUser",
        "UsersRemoveRolesAdminUserMutateRemoveRolesInvalidInput",
        "UsersRemoveRolesAdminUserMutateRemoveRolesOperationError",
    ] = Field(alias="removeRoles", discriminator="typename__")


class UsersRemoveRolesAdminUserMutateRemoveRolesUser(UserSelection):
    typename__: Literal["User"] = Field(alias="__typename")


class UsersRemoveRolesAdminUserMutateRemoveRolesInvalidInput(InvalidInputSelection):
    typename__: Literal["InvalidInput"] = Field(alias="__typename")


class UsersRemoveRolesAdminUserMutateRemoveRolesOperationError(OperationErrorSelection):
    typename__: Literal["OperationError"] = Field(alias="__typename")


UsersRemoveRoles.model_rebuild()
UsersRemoveRolesAdmin.model_rebuild()
UsersRemoveRolesAdminUser.model_rebuild()
UsersRemoveRolesAdminUserMutate.model_rebuild()



"""
Helper functions for connecting to the Quilt Registry.
"""

import json
import os
import platform
import stat
import subprocess
import sys
import time
import typing as T
from importlib import metadata

import boto3
import botocore.session
import requests
from botocore.credentials import (
    CredentialProvider,
    CredentialResolver,
    RefreshableCredentials,
)

from .util import BASE_PATH, QuiltException, get_from_config

AUTH_PATH = BASE_PATH / 'auth.json'
CREDENTIALS_PATH = BASE_PATH / 'credentials.json'
VERSION = metadata.version('quilt3')


def _load_auth():
    try:
        with open(AUTH_PATH, encoding='utf-8') as fd:
            return json.load(fd)
    except FileNotFoundError:
        return {}


def _save_auth(cfg):
    BASE_PATH.mkdir(parents=True, exist_ok=True)
    with open(AUTH_PATH, 'w', encoding='utf-8') as fd:
        AUTH_PATH.chmod(stat.S_IRUSR | stat.S_IWUSR)
        json.dump(cfg, fd)


def _load_credentials():
    try:
        with open(CREDENTIALS_PATH, encoding='utf-8') as fd:
            return json.load(fd)
    except FileNotFoundError:
        return {}


def _save_credentials(creds):
    BASE_PATH.mkdir(parents=True, exist_ok=True)
    with open(CREDENTIALS_PATH, 'w', encoding='utf-8') as fd:
        CREDENTIALS_PATH.chmod(stat.S_IRUSR | stat.S_IWUSR)
        json.dump(creds, fd)


def get_registry_url():
    return get_from_config('registryUrl')


def _update_auth(refresh_token, timeout=None):
    try:
        response = requests.post(
            "%s/api/token" % get_registry_url(),
            timeout=timeout,
            data=dict(
                refresh_token=refresh_token,
            ),
        )
    except requests.exceptions.ConnectionError as ex:
        raise QuiltException("Failed to connect: %s" % ex)

    if response.status_code != requests.codes.ok:
        raise QuiltException("Authentication error: %s" % response.status_code)

    data = response.json()
    error = data.get('error')
    if error is not None:
        raise QuiltException("Failed to log in: %s" % error)

    return dict(
        refresh_token=data['refresh_token'],
        access_token=data['access_token'],
        expires_at=data['expires_at'],
    )


def _handle_response(resp, **kwargs):
    if resp.status_code == requests.codes.unauthorized:
        raise QuiltException("Authentication failed. Run `quilt3 login` again.")
    elif not resp.ok:
        try:
            data = resp.json()
            raise QuiltException(data['message'])
        except ValueError:
            raise QuiltException("Unexpected failure: error %s" % resp.status_code)


def _create_auth(timeout=None):
    """
    Reads the credentials, updates the access token if necessary, and returns it.
    """
    url = get_registry_url()
    contents = _load_auth()
    auth = contents.get(url)

    if auth is not None:
        # If the access token expires within a minute, update it.
        if auth['expires_at'] < time.time() + 60:
            try:
                auth = _update_auth(auth['refresh_token'], timeout)
            except QuiltException as ex:
                raise QuiltException("Failed to update the access token (%s). Run `quilt3 login` again." % ex)
            contents[url] = auth
            _save_auth(contents)

    return auth


def _create_session(auth):
    """
    Creates a session object to be used for `push`, `install`, etc.
    """
    session = requests.Session()
    session.hooks.update(response=_handle_response)
    session.headers.update(
        {
            "Content-Type": "application/json",
            "Accept": "application/json",
            "User-Agent": "quilt-python/%s (%s %s) %s/%s"
            % (
                VERSION,
                platform.system(),
                platform.release(),
                platform.python_implementation(),
                platform.python_version(),
            ),
        }
    )
    if auth is not None:
        session.headers["Authorization"] = "Bearer %s" % auth['access_token']

    return session


_session = None


def get_session(timeout=None):
    """
    Creates a session or returns an existing session.
    """
    global _session
    if _session is None:
        auth = _create_auth(timeout)
        _session = _create_session(auth)

    assert _session is not None

    return _session


def clear_session():
    global _session
    if _session is not None:
        _session.close()
        _session = None


def open_url(url):
    try:
        if sys.platform == 'win32':
            os.startfile(url)  # pylint:disable=E1101
        elif sys.platform == 'darwin':
            with open(os.devnull, 'rb+') as null:
                subprocess.check_call(['open', url], stdin=null, stdout=null, stderr=null)
        else:
            with open(os.devnull, 'rb+') as null:
                subprocess.check_call(['xdg-open', url], stdin=null, stdout=null, stderr=null)
    except Exception as ex:  # pylint:disable=W0703
        print("Failed to launch the browser: %s" % ex)


def login():
    """
    Authenticate to your Quilt stack and assume the role assigned to you by
    your stack administrator. Not required if you have existing AWS credentials.

    Launches a web browser and asks the user for a token.
    """
    registry_url = get_registry_url()
    if registry_url is None:
        raise QuiltException(
            "You attempted to authenticate to a Quilt catalog, but your home catalog is "
            "currently set to None. Please first specify your home catalog by running "
            "\"quilt3.config('$URL')\", replacing '$URL' with your catalog homepage."
        )

    login_url = "%s/login" % get_registry_url()

    print("Launching a web browser...")
    print("If that didn't work, please visit the following URL: %s" % login_url)

    open_url(login_url)

    print()
    refresh_token = input("Enter the code from the webpage: ")

    login_with_token(refresh_token)


def login_with_token(refresh_token):
    """
    Authenticate using an existing token.
    """
    # Get an access token and a new refresh token.
    auth = _update_auth(refresh_token)

    url = get_registry_url()
    contents = _load_auth()
    contents[url] = auth
    _save_auth(contents)

    clear_session()

    # use registry-provided credentials
    _refresh_credentials()


def logout():
    """
    Do not use Quilt credentials. Useful if you have existing AWS credentials.
    """
    # TODO revoke refresh token (without logging out of web sessions)
    if _load_auth() or _load_credentials():
        _save_auth({})
        _save_credentials({})
    else:
        print("Already logged out.")

    clear_session()


def _refresh_credentials():
    session = get_session()
    creds = session.get("{url}/api/auth/get_credentials".format(url=get_registry_url())).json()
    result = {
        'access_key': creds['AccessKeyId'],
        'secret_key': creds['SecretAccessKey'],
        'token': creds['SessionToken'],
        'expiry_time': creds['Expiration'],
    }
    _save_credentials(result)
    return result


def logged_in():
    """
    Return catalog URL if Quilt client is authenticated. Otherwise
    return `None`.
    """
    url = get_registry_url()
    if url in _load_auth():
        return get_from_config('navigator_url')


class QuiltProvider(CredentialProvider):
    METHOD = 'quilt-registry'
    CANONICAL_NAME = 'QuiltRegistry'

    def __init__(self, credentials):
        super().__init__()
        self._credentials = credentials

    def load(self):
        creds = RefreshableCredentials.create_from_metadata(
            metadata=self._credentials,
            method=self.METHOD,
            refresh_using=_refresh_credentials,
        )

        return creds


def create_botocore_session(*, credentials: T.Optional[dict] = None) -> botocore.session.Session:
    botocore_session = botocore.session.get_session()

    # If we have saved credentials, use them. Otherwise, create a normal Boto session.
    if credentials is None:
        credentials = _load_credentials()
    if credentials:
        provider = QuiltProvider(credentials)
        resolver = CredentialResolver([provider])
        botocore_session.register_component('credential_provider', resolver)

    return botocore_session


def get_boto3_session(*, fallback: bool = True) -> boto3.Session:
    """
    Return a Boto3 session with Quilt stack credentials and AWS region.
    In case of no Quilt credentials found, return a "normal" Boto3 session if `fallback` is `True`,
    otherwise raise a `QuiltException`.

    > Note: you need to call `quilt3.config("https://your-catalog-homepage/")` to have region set on the session,
    if you previously called it in quilt3 < 6.1.0.
    """
    if not (credentials := _load_credentials()):
        if fallback:
            return boto3.Session()
        raise QuiltException("No Quilt credentials found.")
    return boto3.Session(
        botocore_session=create_botocore_session(credentials=credentials),
        region_name=get_from_config("region"),
    )



# Generated by ariadne-codegen
# Source: queries.graphql

from typing import Optional

from pydantic import Field

from .base_model import BaseModel
from .fragments import SsoConfigSelection


class SsoConfigGet(BaseModel):
    admin: "SsoConfigGetAdmin"


class SsoConfigGetAdmin(BaseModel):
    sso_config: Optional["SsoConfigGetAdminSsoConfig"] = Field(alias="ssoConfig")


class SsoConfigGetAdminSsoConfig(SsoConfigSelection):
    pass


SsoConfigGet.model_rebuild()
SsoConfigGetAdmin.model_rebuild()



# Generated by ariadne-codegen
# Source: queries.graphql

from typing import Literal, Union

from pydantic import Field

from .base_model import BaseModel
from .fragments import InvalidInputSelection, OperationErrorSelection, UserSelection


class UsersCreate(BaseModel):
    admin: "UsersCreateAdmin"


class UsersCreateAdmin(BaseModel):
    user: "UsersCreateAdminUser"


class UsersCreateAdminUser(BaseModel):
    create: Union[
        "UsersCreateAdminUserCreateUser",
        "UsersCreateAdminUserCreateInvalidInput",
        "UsersCreateAdminUserCreateOperationError",
    ] = Field(discriminator="typename__")


class UsersCreateAdminUserCreateUser(UserSelection):
    typename__: Literal["User"] = Field(alias="__typename")


class UsersCreateAdminUserCreateInvalidInput(InvalidInputSelection):
    typename__: Literal["InvalidInput"] = Field(alias="__typename")


class UsersCreateAdminUserCreateOperationError(OperationErrorSelection):
    typename__: Literal["OperationError"] = Field(alias="__typename")


UsersCreate.model_rebuild()
UsersCreateAdmin.model_rebuild()
UsersCreateAdminUser.model_rebuild()



"""
APIs for Quilt administrators. 'Registry' refers to Quilt stack backend services, including identity management.
"""

# This wraps code generated by ariadne-codegen to provide a more user-friendly API.

from . import buckets, roles, sso_config, tabulator, users
from .exceptions import (
    BucketNotFoundError,
    Quilt3AdminError,
    UserNotFoundError,
)
from .types import Bucket, ManagedRole, SSOConfig, TabulatorTable, UnmanagedRole, User



import datetime
import json
import os
import pathlib
import re
import warnings
from collections import OrderedDict
from urllib.parse import (
    parse_qs,
    quote,
    unquote,
    urlencode,
    urlparse,
    urlunparse,
)
from urllib.request import url2pathname

import requests

# Third-Party
import yaml
from platformdirs import user_cache_dir, user_data_dir


def get_bool_from_env(var_name: str):
    return os.getenv(var_name, '').lower() == 'true'


APP_NAME = "Quilt"
APP_AUTHOR = "QuiltData"
BASE_DIR = user_data_dir(APP_NAME, APP_AUTHOR)
BASE_PATH = pathlib.Path(BASE_DIR)
CACHE_PATH = pathlib.Path(user_cache_dir(APP_NAME, APP_AUTHOR)) / "v0"
TEMPFILE_DIR_PATH = BASE_PATH / "tempfiles"
CONFIG_PATH = BASE_PATH / 'config.yml'
OPEN_DATA_URL = "https://open.quiltdata.com"

PACKAGE_NAME_FORMAT = r"^[\w-]+/[\w-]+$"
DISABLE_TQDM = get_bool_from_env('QUILT_MINIMIZE_STDOUT')
PACKAGE_UPDATE_POLICY = {'incoming', 'existing'}
IS_CACHE_ENABLED = not get_bool_from_env('QUILT_DISABLE_CACHE')


# CONFIG_TEMPLATE
# Must contain every permitted config key, as well as their default values (which can be 'null'/None).
# Comments are retained and added to local config, unless overridden by autoconfig via `api.config(<url>)`
CONFIG_TEMPLATE = """
# Quilt3 configuration file

# navigator_url: <url string, default: null>
#
# Used for autoconfiguration
# navigator_url: https://example.com
navigator_url:

# default_local_registry: <url string, default: local data directory (platform dependent)>
# default target registry for operations like install and build
default_local_registry: "{}"

# default_remote_registry: <url string, default: null>
# default target for operations like push and browse
default_remote_registry:

# default_install_location: <url string, default: null>
# default filesystem target for the install operation
default_install_location:

# Identity service URL
registryUrl:

# Disable anonymous usage metrics
telemetry_disabled: false

# S3 Proxy
s3Proxy:

# API Gateway endpoint (e.g., for search)
apiGatewayEndpoint:

# Binary API Gateway endpoint (e.g., for preview)
binaryApiGatewayEndpoint:

default_registry_version: 1

# AWS region
region:

""".format(BASE_PATH.as_uri() + '/packages')


def get_pos_int_from_env(var_name):
    val = os.getenv(var_name)
    if val:
        try:
            val = int(val)
        except ValueError:
            val = None
        if val is None or val <= 0:
            raise ValueError(f'{var_name} must be a positive integer')
        return val


class QuiltException(Exception):
    def __init__(self, message, **kwargs):
        # We use NewError("Prefix: " + str(error)) a lot.
        # To be consistent across Python 2.7 and 3.x:
        # 1) This `super` call must exist, or 2.7 will have no text for str(error)
        # 2) This `super` call must have only one argument (the message) or str(error) will be a repr of args
        super().__init__(message)
        self.message = message
        for k, v in kwargs.items():
            setattr(self, k, v)


class QuiltConflictException(QuiltException):
    pass


class RemovedInQuilt4Warning(FutureWarning):
    pass


class URLParseError(ValueError):
    pass


class PhysicalKey:
    __slots__ = ('bucket', 'path', 'version_id')

    def __init__(self, bucket, path, version_id):
        """
        For internal use only; call from_path or from_url instead.
        """
        assert bucket is None or isinstance(bucket, str)
        assert isinstance(path, str)
        assert version_id is None or isinstance(version_id, str)

        if bucket is None:
            assert path is not None, "Local keys must have a path"
            assert version_id is None, "Local keys cannot have a version ID"
            if os.name == 'nt':
                assert '\\' not in path, "Paths must use / as a separator"

        self.bucket = bucket
        self.path = path
        self.version_id = version_id

    @classmethod
    def from_url(cls, url):
        parsed = urlparse(url)

        if parsed.scheme == 's3':
            if not parsed.netloc:
                raise URLParseError("Missing bucket")
            bucket = parsed.netloc
            assert not parsed.path or parsed.path.startswith('/')
            path = unquote(parsed.path)[1:]
            # Parse the version ID the way the Java SDK does:
            # https://github.com/aws/aws-sdk-java/blob/master/aws-java-sdk-s3/src/main/java/com/amazonaws/services/s3/AmazonS3URI.java#L192
            query = parse_qs(parsed.query)
            version_id = query.pop('versionId', [None])[0]
            if query:
                raise URLParseError(f"Unexpected S3 query string: {parsed.query!r}")
            return cls(bucket, path, version_id)
        elif parsed.scheme == 'file':
            if parsed.netloc not in ('', 'localhost'):
                raise URLParseError("Unexpected hostname")
            if not parsed.path:
                raise URLParseError("Missing path")
            if not parsed.path.startswith('/'):
                raise URLParseError("Relative paths are not allowed")
            if parsed.query:
                raise URLParseError("Unexpected query")
            path = url2pathname(parsed.path)
            if parsed.path.endswith('/') and not path.endswith(os.path.sep):
                # On Windows, url2pathname loses the trailing `/`.
                path += os.path.sep
            return cls.from_path(path)
        else:
            raise URLParseError(f"Unexpected scheme: {parsed.scheme!r}")

    @classmethod
    def from_path(cls, path):
        path = os.fspath(path)
        new_path = os.path.realpath(path)
        # Use '/' as the path separator.
        if os.path.sep != '/':
            new_path = new_path.replace(os.path.sep, '/')
        # Add back a trailing '/' if the original path has it.
        if path.endswith(os.path.sep) or (os.path.altsep is not None and path.endswith(os.path.altsep)):
            new_path += '/'
        return cls(None, new_path, None)

    def is_local(self):
        return self.bucket is None

    def join(self, rel_path):
        if self.version_id is not None:
            raise ValueError('Cannot append paths to URLs with a version ID')

        if os.name == 'nt' and '\\' in rel_path:
            raise ValueError("Paths must use / as a separator")

        if self.path:
            new_path = self.path.rstrip('/') + '/' + rel_path.lstrip('/')
        else:
            new_path = rel_path.lstrip('/')
        return PhysicalKey(self.bucket, new_path, None)

    def basename(self):
        return self.path.rsplit('/', 1)[-1]

    def __eq__(self, other):
        return (
            isinstance(other, self.__class__)
            and self.bucket == other.bucket
            and self.path == other.path
            and self.version_id == other.version_id
        )

    def __repr__(self):
        return f'{self.__class__.__name__}({self.bucket!r}, {self.path!r}, {self.version_id!r})'

    def __str__(self):
        if self.bucket is None:
            return pathlib.PurePath(self.path).as_uri()
        else:
            if self.version_id is None:
                params = {}
            else:
                params = {'versionId': self.version_id}
            return urlunparse(('s3', self.bucket, quote('/' + self.path), None, urlencode(params), None))


def fix_url(url):
    """Convert non-URL paths to file:// URLs"""
    # If it has a scheme, we assume it's a URL.
    # On Windows, we ignore schemes that look like drive letters, e.g. C:/users/foo
    if not url:
        raise ValueError("Empty URL")

    url = str(url)

    parsed = urlparse(url)
    if parsed.scheme and not os.path.splitdrive(url)[0]:
        return url

    # `expanduser()` expands any leading "~" or "~user" path components, as a user convenience
    # `resolve()` _tries_ to make the URI absolute - but doesn't guarantee anything.
    # In particular, on Windows, non-existent files won't be resolved.
    # `absolute()` makes the URI absolute, though it can still contain '..'
    fixed_url = pathlib.Path(url).expanduser().resolve().absolute().as_uri()

    # pathlib likes to remove trailing slashes, so add it back if needed.
    if url[-1:] in (os.sep, os.altsep) and not fixed_url.endswith('/'):
        fixed_url += '/'

    return fixed_url


def extract_file_extension(file_path_or_url):
    """
    Extract the file extension if it exists.

    Args:
        file_path_or_url: The path to the file. Type can can be anything that pathlib.Path understands.

    Returns:
        File extension without the period, i.e. ("txt" not ".txt"). None if the path does not have an extension.
    """
    p = pathlib.Path(file_path_or_url)
    if len(p.suffix) > 0:
        return p.suffix[1:]
    else:
        return None


def read_yaml(yaml_stream):
    try:
        if isinstance(yaml_stream, pathlib.Path):
            with yaml_stream.open(mode='r') as stream:
                return yaml.safe_load(stream)
        return yaml.safe_load(yaml_stream)
    except yaml.YAMLError as error:
        raise QuiltException(str(error), original_error=error)


def write_yaml(data, yaml_path, keep_backup=False):
    """Write `data` to `yaml_path`

    :param data: Any yaml-serializable data
    :param yaml_path: Destination. Can be a string or pathlib path.
    :param keep_backup: If set, a timestamped backup will be kept in the same dir.
    """
    path = pathlib.Path(yaml_path)
    now = datetime.datetime.now(datetime.timezone.utc).strftime("%Y%m%dT%H%M%S.%fZ")  # ISO 8601 'basic format'
    backup_path = path.with_name(path.name + '.backup.' + now)

    try:
        if path.exists():
            # TODO: use something from tempfile to make sure backup_path doesn't exist.
            path.replace(backup_path)
        path.parent.mkdir(parents=True, exist_ok=True)
        with path.open('w') as config_file:
            yaml.dump(data, config_file)
    except Exception:  # intentionally wide catch -- reraised immediately.
        if backup_path.exists():
            backup_path.replace(path)
        raise

    if backup_path.exists() and not keep_backup:
        backup_path.unlink()


def validate_url(url):
    """A URL must have scheme and host, at minimum."""
    parsed_url = urlparse(url)

    # require scheme and host at minimum, like config_path'http://foo'
    if not all((parsed_url.scheme, parsed_url.netloc)):
        raise QuiltException("Invalid URL -- Requires at least scheme and host: {}".format(url))
    try:
        parsed_url.port
    except ValueError:
        raise QuiltException("Invalid URL -- Port must be a number: {}".format(url))


# Although displaying the config may seem not to warrant a class, it's pretty important
# for good UX. A lot of points were considered in making this -- retaining order,
# user's usage in an interpreted environment like Jupyter, and keeping the displayed
# information concise.  Given the limitations of the other options, making a class with
# custom repr panned out to be the best (and shortest) option.
class QuiltConfig(OrderedDict):
    def __init__(self, filepath, *args, **kwargs):
        self.filepath = pathlib.Path(filepath)
        super().__init__(*args, **kwargs)

    def __setitem__(self, key, value):
        # Per chat in #engineering 4-5-19, strip navigator_url of trailing slash.
        # Ideally, we should do that kind of thing in one cohesive spot.
        # This is a good spot.
        if key == 'navigator_url' and value:
            if not isinstance(value, str):
                raise ValueError("Expected a string for config key {!r}, but got {!r}".format(key, value))
            value = value.strip().rstrip('/')
        # Similar activity, moved from api.config() to here.
        if isinstance(key, str) and key.endswith('_url'):
            if value:
                validate_url(value)
        super().__setitem__(key, value)

    # TODO: Make an _html_repr_ for nicer Notebook display
    def __repr__(self):
        return "<{} at {!r} {}>".format(type(self).__name__, str(self.filepath), json.dumps(self, indent=4))


def validate_package_name(name):
    """Verify that a package name is two alphanumeric strings separated by a slash."""
    if not re.match(PACKAGE_NAME_FORMAT, name):
        raise QuiltException(f"Invalid package name: {name}.")


def configure_from_url(catalog_url):
    """Read configuration settings from a Quilt catalog"""
    config_template = read_yaml(CONFIG_TEMPLATE)
    # Clean up and validate catalog url
    catalog_url = catalog_url.rstrip('/')
    validate_url(catalog_url)

    # Get the new config
    config_url = catalog_url + '/config.json'

    response = requests.get(config_url)
    if not response.ok:
        message = "An HTTP Error ({code}) occurred: {reason}"
        raise QuiltException(
            message.format(code=response.status_code, reason=response.reason),
            response=response,
        )

    # QuiltConfig may perform some validation and value scrubbing.
    new_config = QuiltConfig('', response.json())

    # 'navigator_url' needs to be renamed, the term is outdated.
    if not new_config.get('navigator_url'):
        new_config['navigator_url'] = catalog_url

    # Use our template + their configured values, keeping our comments.
    for key, value in new_config.items():
        if key not in config_template:
            continue
        config_template[key] = value
    write_yaml(config_template, CONFIG_PATH, keep_backup=True)
    return config_template


def config_exists():
    """
    Returns True if a config file (config.yml) is installed.
    """
    return CONFIG_PATH.exists()


def user_is_configured_to_custom_stack():
    """Look at the users stack to see if they have configured to their own stack."""
    configured_nav_url = get_from_config("navigator_url")
    return configured_nav_url is not None


def configure_from_default():
    """
    Try to configure to the default (public) Quilt stack.
    If reading from the public stack fails, warn the user
    and save an empty template.
    """
    try:
        local_config = configure_from_url(OPEN_DATA_URL)
    except requests.exceptions.ConnectionError:
        msg = f"Failed to connect to {OPEN_DATA_URL}."
        msg += "Some features will not work without a"
        msg += "valid configuration."
        warnings.warn(msg)
        config_template = read_yaml(CONFIG_TEMPLATE)
        write_yaml(config_template, CONFIG_PATH, keep_backup=True)
        local_config = config_template
    return local_config


def load_config():
    """
    Read the local config using defaults from CONFIG_TEMPLATE.
    """
    local_config = read_yaml(CONFIG_TEMPLATE)
    try:
        local_config.update(read_yaml(CONFIG_PATH))
    except FileNotFoundError:
        pass
    return local_config


def get_from_config(key):
    return load_config().get(key)


def get_install_location():
    loc = get_from_config('default_install_location')
    if loc is None:
        loc = get_from_config('default_local_registry').rstrip('/')
    return loc


def set_config_value(key, value):
    # Use local configuration (or defaults)
    local_config = load_config()
    local_config[key] = value
    write_yaml(local_config, CONFIG_PATH)


def quiltignore_filter(paths, ignore, url_scheme):
    """Given a list of paths, filter out the paths which are captured by the
    given ignore rules.

    Args:
        paths (list): a list or iterable of paths
        ignore (path): a path to the file defining ignore rules, in Unix shell
            style wildcard format
        url_scheme (str): the URL scheme, only the "file" scheme is currently
            supported
    """
    ignore_rules = ignore.read_text('utf-8').split("\n")
    ignore_rules = ['*/' + rule for rule in ignore_rules if rule]

    if url_scheme == 'file':
        from fnmatch import fnmatch

        files, dirs = set(), set()
        for path in paths:
            if path.is_file():
                files.add(path)
            else:
                dirs.add(path)

        filtered_dirs = dirs.copy()
        for ignore_rule in ignore_rules:
            for pkg_dir in filtered_dirs.copy():
                # copy git behavior --- git matches paths and directories equivalently.
                # e.g. both foo and foo/ will match the ignore rule "foo"
                # but only foo/ will match the ignore rule "foo/"
                if fnmatch(pkg_dir.as_posix() + "/", ignore_rule) or fnmatch(pkg_dir.as_posix(), ignore_rule):
                    files = {n for n in files if pkg_dir not in n.parents}
                    dirs = dirs - {pkg_dir}

            files = {n for n in files if not fnmatch(n, ignore_rule)}

        return files.union(dirs)
    else:
        raise NotImplementedError


def validate_key(key):
    """
    Verify that a file path or S3 path does not contain any '.' or '..' separators or files.
    """
    if key is None or key == '':
        raise QuiltException(f"Invalid key {key!r}. A package entry key cannot be empty.")

    for part in key.split('/'):
        if part in ('', '.', '..'):
            raise QuiltException(
                f"Invalid key {key!r}. "
                f"A package entry key cannot contain a file or folder named '.' or '..' in its path."
            )


def catalog_s3_url(catalog_url, s3_url):
    """
    Generate a URL to the Quilt catalog page for an object in S3
    """
    if s3_url is None:
        return catalog_url

    pk = PhysicalKey.from_url(s3_url)
    if pk.is_local():
        raise QuiltException("Not an S3 URL")

    url = f"{catalog_url}/b/{quote(pk.bucket)}"

    if pk.path:
        url += f"/tree/{quote(pk.path)}"

        # Ignore version_id if path is empty (e.g., s3://<bucket>)
        if pk.version_id is not None:
            params = {'version': pk.version_id}
            url += f"?{urlencode(params)}"
    return url


def catalog_package_url(catalog_url, bucket, package_name, package_timestamp="latest", tree=True):
    """
    Generate a URL to the Quilt catalog page of a package. By default will go to the latest version of the package,
    but the user can pass in the appropriate timestamp to go to a different version.
    Disabling tree by passing `tree=False` will generate a package URL without tree path.

    Note: There is currently no good way to generate the URL given a specific tophash
    """
    assert bucket is not None, "The bucket parameter must not be None"
    assert package_name is not None, "The package_name parameter must not be None"
    validate_package_name(package_name)

    package_url = f"{catalog_url}/b/{bucket}/packages/{package_name}"
    if tree:
        package_url = package_url + f"/tree/{package_timestamp}"
    return package_url



# Generated by ariadne-codegen
# Source: queries.graphql

from typing import List, Optional

from pydantic import Field

from .base_model import BaseModel


class BucketTabulatorTablesList(BaseModel):
    bucket_config: Optional["BucketTabulatorTablesListBucketConfig"] = Field(alias="bucketConfig")


class BucketTabulatorTablesListBucketConfig(BaseModel):
    tabulator_tables: List["BucketTabulatorTablesListBucketConfigTabulatorTables"] = Field(alias="tabulatorTables")


class BucketTabulatorTablesListBucketConfigTabulatorTables(BaseModel):
    name: str
    config: str


BucketTabulatorTablesList.model_rebuild()
BucketTabulatorTablesListBucketConfig.model_rebuild()



# Generated by ariadne-codegen
# Source: ../../../shared/graphql/schema.graphql

from datetime import datetime
from typing import Annotated, Any, List, Optional, Union

from pydantic import Field, PlainSerializer

from .base_model import BaseModel, Upload


class BucketAddInput(BaseModel):
    name: str
    title: str
    icon_url: Optional[str] = Field(alias="iconUrl", default=None)
    description: Optional[str] = None
    linked_data: Optional[Any] = Field(alias="linkedData", default=None)
    overview_url: Optional[str] = Field(alias="overviewUrl", default=None)
    tags: Optional[List[str]] = None
    relevance_score: Optional[int] = Field(alias="relevanceScore", default=None)
    sns_notification_arn: Optional[str] = Field(alias="snsNotificationArn", default=None)
    scanner_parallel_shards_depth: Optional[int] = Field(alias="scannerParallelShardsDepth", default=None)
    skip_meta_data_indexing: Optional[bool] = Field(alias="skipMetaDataIndexing", default=None)
    file_extensions_to_index: Optional[List[str]] = Field(alias="fileExtensionsToIndex", default=None)
    index_content_bytes: Optional[int] = Field(alias="indexContentBytes", default=None)
    delay_scan: Optional[bool] = Field(alias="delayScan", default=None)
    browsable: Optional[bool] = None
    prefixes: Optional[List[str]] = None


class BucketUpdateInput(BaseModel):
    title: str
    icon_url: Optional[str] = Field(alias="iconUrl", default=None)
    description: Optional[str] = None
    linked_data: Optional[Any] = Field(alias="linkedData", default=None)
    overview_url: Optional[str] = Field(alias="overviewUrl", default=None)
    tags: Optional[List[str]] = None
    relevance_score: Optional[int] = Field(alias="relevanceScore", default=None)
    sns_notification_arn: Optional[str] = Field(alias="snsNotificationArn", default=None)
    scanner_parallel_shards_depth: Optional[int] = Field(alias="scannerParallelShardsDepth", default=None)
    skip_meta_data_indexing: Optional[bool] = Field(alias="skipMetaDataIndexing", default=None)
    file_extensions_to_index: Optional[List[str]] = Field(alias="fileExtensionsToIndex", default=None)
    index_content_bytes: Optional[int] = Field(alias="indexContentBytes", default=None)
    browsable: Optional[bool] = None
    prefixes: Optional[List[str]] = None


class UserInput(BaseModel):
    name: str
    email: str
    role: str
    extra_roles: Optional[List[str]] = Field(alias="extraRoles", default=None)



"""Quilt API"""

# Suppress numpy warnings
import warnings

warnings.filterwarnings("ignore", message="numpy.dtype size changed")  # noqa: E402

import importlib.metadata

__version__ = importlib.metadata.version(__name__)

from . import admin, hooks
from .api import (
    config,
    copy,
    delete_package,
    disable_telemetry,
    list_package_versions,
    list_packages,
    search,
)
from .bucket import Bucket
from .imports import start_data_package_loader
from .packages import Package
from .session import get_boto3_session, logged_in, login, logout

start_data_package_loader()



# Generated by ariadne-codegen

from .base_client import BaseClient
from .base_model import BaseModel, Upload
from .bucket_add import (
    BucketAdd,
    BucketAddBucketAddBucketAddSuccess,
    BucketAddBucketAddBucketAddSuccessBucketConfig,
    BucketAddBucketAddBucketAlreadyAdded,
    BucketAddBucketAddBucketDoesNotExist,
    BucketAddBucketAddBucketFileExtensionsToIndexInvalid,
    BucketAddBucketAddBucketIndexContentBytesInvalid,
    BucketAddBucketAddInsufficientPermissions,
    BucketAddBucketAddNotificationConfigurationError,
    BucketAddBucketAddNotificationTopicNotFound,
    BucketAddBucketAddSnsInvalid,
    BucketAddBucketAddSubscriptionInvalid,
)
from .bucket_get import BucketGet, BucketGetBucketConfig
from .bucket_remove import (
    BucketRemove,
    BucketRemoveBucketRemoveBucketNotFound,
    BucketRemoveBucketRemoveBucketRemoveSuccess,
    BucketRemoveBucketRemoveIndexingInProgress,
)
from .bucket_tabulator_table_rename import (
    BucketTabulatorTableRename,
    BucketTabulatorTableRenameAdmin,
    BucketTabulatorTableRenameAdminBucketRenameTabulatorTableBucketConfig,
    BucketTabulatorTableRenameAdminBucketRenameTabulatorTableInvalidInput,
    BucketTabulatorTableRenameAdminBucketRenameTabulatorTableOperationError,
)
from .bucket_tabulator_table_set import (
    BucketTabulatorTableSet,
    BucketTabulatorTableSetAdmin,
    BucketTabulatorTableSetAdminBucketSetTabulatorTableBucketConfig,
    BucketTabulatorTableSetAdminBucketSetTabulatorTableInvalidInput,
    BucketTabulatorTableSetAdminBucketSetTabulatorTableOperationError,
)
from .bucket_tabulator_tables_list import (
    BucketTabulatorTablesList,
    BucketTabulatorTablesListBucketConfig,
    BucketTabulatorTablesListBucketConfigTabulatorTables,
)
from .bucket_update import (
    BucketUpdate,
    BucketUpdateBucketUpdateBucketFileExtensionsToIndexInvalid,
    BucketUpdateBucketUpdateBucketIndexContentBytesInvalid,
    BucketUpdateBucketUpdateBucketNotFound,
    BucketUpdateBucketUpdateBucketUpdateSuccess,
    BucketUpdateBucketUpdateBucketUpdateSuccessBucketConfig,
    BucketUpdateBucketUpdateInsufficientPermissions,
    BucketUpdateBucketUpdateNotificationConfigurationError,
    BucketUpdateBucketUpdateNotificationTopicNotFound,
    BucketUpdateBucketUpdateSnsInvalid,
)
from .buckets_list import BucketsList, BucketsListBucketConfigs
from .client import Client
from .fragments import (
    BucketConfigSelection,
    InvalidInputSelection,
    InvalidInputSelectionErrors,
    ManagedRoleSelection,
    OperationErrorSelection,
    SsoConfigSelection,
    SsoConfigSelectionUploader,
    UnmanagedRoleSelection,
    UserSelection,
    UserSelectionExtraRolesManagedRole,
    UserSelectionExtraRolesUnmanagedRole,
    UserSelectionRoleManagedRole,
    UserSelectionRoleUnmanagedRole,
)
from .input_types import BucketAddInput, BucketUpdateInput, UserInput
from .roles_list import (
    RolesList,
    RolesListRolesManagedRole,
    RolesListRolesUnmanagedRole,
)
from .sso_config_get import SsoConfigGet, SsoConfigGetAdmin, SsoConfigGetAdminSsoConfig
from .sso_config_set import (
    SsoConfigSet,
    SsoConfigSetAdmin,
    SsoConfigSetAdminSetSsoConfigInvalidInput,
    SsoConfigSetAdminSetSsoConfigOperationError,
    SsoConfigSetAdminSetSsoConfigSsoConfig,
)
from .tabulator_get_open_query import TabulatorGetOpenQuery, TabulatorGetOpenQueryAdmin
from .tabulator_set_open_query import (
    TabulatorSetOpenQuery,
    TabulatorSetOpenQueryAdmin,
    TabulatorSetOpenQueryAdminSetTabulatorOpenQuery,
)
from .users_add_roles import (
    UsersAddRoles,
    UsersAddRolesAdmin,
    UsersAddRolesAdminUser,
    UsersAddRolesAdminUserMutate,
    UsersAddRolesAdminUserMutateAddRolesInvalidInput,
    UsersAddRolesAdminUserMutateAddRolesOperationError,
    UsersAddRolesAdminUserMutateAddRolesUser,
)
from .users_create import (
    UsersCreate,
    UsersCreateAdmin,
    UsersCreateAdminUser,
    UsersCreateAdminUserCreateInvalidInput,
    UsersCreateAdminUserCreateOperationError,
    UsersCreateAdminUserCreateUser,
)
from .users_delete import (
    UsersDelete,
    UsersDeleteAdmin,
    UsersDeleteAdminUser,
    UsersDeleteAdminUserMutate,
    UsersDeleteAdminUserMutateDeleteInvalidInput,
    UsersDeleteAdminUserMutateDeleteOk,
    UsersDeleteAdminUserMutateDeleteOperationError,
)
from .users_get import UsersGet, UsersGetAdmin, UsersGetAdminUser, UsersGetAdminUserGet
from .users_list import (
    UsersList,
    UsersListAdmin,
    UsersListAdminUser,
    UsersListAdminUserList,
)
from .users_remove_roles import (
    UsersRemoveRoles,
    UsersRemoveRolesAdmin,
    UsersRemoveRolesAdminUser,
    UsersRemoveRolesAdminUserMutate,
    UsersRemoveRolesAdminUserMutateRemoveRolesInvalidInput,
    UsersRemoveRolesAdminUserMutateRemoveRolesOperationError,
    UsersRemoveRolesAdminUserMutateRemoveRolesUser,
)
from .users_reset_password import (
    UsersResetPassword,
    UsersResetPasswordAdmin,
    UsersResetPasswordAdminUser,
    UsersResetPasswordAdminUserMutate,
    UsersResetPasswordAdminUserMutateResetPasswordInvalidInput,
    UsersResetPasswordAdminUserMutateResetPasswordOk,
    UsersResetPasswordAdminUserMutateResetPasswordOperationError,
)
from .users_set_active import (
    UsersSetActive,
    UsersSetActiveAdmin,
    UsersSetActiveAdminUser,
    UsersSetActiveAdminUserMutate,
    UsersSetActiveAdminUserMutateSetActiveInvalidInput,
    UsersSetActiveAdminUserMutateSetActiveOperationError,
    UsersSetActiveAdminUserMutateSetActiveUser,
)
from .users_set_admin import (
    UsersSetAdmin,
    UsersSetAdminAdmin,
    UsersSetAdminAdminUser,
    UsersSetAdminAdminUserMutate,
    UsersSetAdminAdminUserMutateSetAdminInvalidInput,
    UsersSetAdminAdminUserMutateSetAdminOperationError,
    UsersSetAdminAdminUserMutateSetAdminUser,
)
from .users_set_email import (
    UsersSetEmail,
    UsersSetEmailAdmin,
    UsersSetEmailAdminUser,
    UsersSetEmailAdminUserMutate,
    UsersSetEmailAdminUserMutateSetEmailInvalidInput,
    UsersSetEmailAdminUserMutateSetEmailOperationError,
    UsersSetEmailAdminUserMutateSetEmailUser,
)
from .users_set_role import (
    UsersSetRole,
    UsersSetRoleAdmin,
    UsersSetRoleAdminUser,
    UsersSetRoleAdminUserMutate,
    UsersSetRoleAdminUserMutateSetRoleInvalidInput,
    UsersSetRoleAdminUserMutateSetRoleOperationError,
    UsersSetRoleAdminUserMutateSetRoleUser,
)

__all__ = [
    "BaseClient",
    "BaseModel",
    "BucketAdd",
    "BucketAddBucketAddBucketAddSuccess",
    "BucketAddBucketAddBucketAddSuccessBucketConfig",
    "BucketAddBucketAddBucketAlreadyAdded",
    "BucketAddBucketAddBucketDoesNotExist",
    "BucketAddBucketAddBucketFileExtensionsToIndexInvalid",
    "BucketAddBucketAddBucketIndexContentBytesInvalid",
    "BucketAddBucketAddInsufficientPermissions",
    "BucketAddBucketAddNotificationConfigurationError",
    "BucketAddBucketAddNotificationTopicNotFound",
    "BucketAddBucketAddSnsInvalid",
    "BucketAddBucketAddSubscriptionInvalid",
    "BucketAddInput",
    "BucketConfigSelection",
    "BucketGet",
    "BucketGetBucketConfig",
    "BucketRemove",
    "BucketRemoveBucketRemoveBucketNotFound",
    "BucketRemoveBucketRemoveBucketRemoveSuccess",
    "BucketRemoveBucketRemoveIndexingInProgress",
    "BucketTabulatorTableRename",
    "BucketTabulatorTableRenameAdmin",
    "BucketTabulatorTableRenameAdminBucketRenameTabulatorTableBucketConfig",
    "BucketTabulatorTableRenameAdminBucketRenameTabulatorTableInvalidInput",
    "BucketTabulatorTableRenameAdminBucketRenameTabulatorTableOperationError",
    "BucketTabulatorTableSet",
    "BucketTabulatorTableSetAdmin",
    "BucketTabulatorTableSetAdminBucketSetTabulatorTableBucketConfig",
    "BucketTabulatorTableSetAdminBucketSetTabulatorTableInvalidInput",
    "BucketTabulatorTableSetAdminBucketSetTabulatorTableOperationError",
    "BucketTabulatorTablesList",
    "BucketTabulatorTablesListBucketConfig",
    "BucketTabulatorTablesListBucketConfigTabulatorTables",
    "BucketUpdate",
    "BucketUpdateBucketUpdateBucketFileExtensionsToIndexInvalid",
    "BucketUpdateBucketUpdateBucketIndexContentBytesInvalid",
    "BucketUpdateBucketUpdateBucketNotFound",
    "BucketUpdateBucketUpdateBucketUpdateSuccess",
    "BucketUpdateBucketUpdateBucketUpdateSuccessBucketConfig",
    "BucketUpdateBucketUpdateInsufficientPermissions",
    "BucketUpdateBucketUpdateNotificationConfigurationError",
    "BucketUpdateBucketUpdateNotificationTopicNotFound",
    "BucketUpdateBucketUpdateSnsInvalid",
    "BucketUpdateInput",
    "BucketsList",
    "BucketsListBucketConfigs",
    "Client",
    "InvalidInputSelection",
    "InvalidInputSelectionErrors",
    "ManagedRoleSelection",
    "OperationErrorSelection",
    "RolesList",
    "RolesListRolesManagedRole",
    "RolesListRolesUnmanagedRole",
    "SsoConfigGet",
    "SsoConfigGetAdmin",
    "SsoConfigGetAdminSsoConfig",
    "SsoConfigSelection",
    "SsoConfigSelectionUploader",
    "SsoConfigSet",
    "SsoConfigSetAdmin",
    "SsoConfigSetAdminSetSsoConfigInvalidInput",
    "SsoConfigSetAdminSetSsoConfigOperationError",
    "SsoConfigSetAdminSetSsoConfigSsoConfig",
    "TabulatorGetOpenQuery",
    "TabulatorGetOpenQueryAdmin",
    "TabulatorSetOpenQuery",
    "TabulatorSetOpenQueryAdmin",
    "TabulatorSetOpenQueryAdminSetTabulatorOpenQuery",
    "UnmanagedRoleSelection",
    "Upload",
    "UserInput",
    "UserSelection",
    "UserSelectionExtraRolesManagedRole",
    "UserSelectionExtraRolesUnmanagedRole",
    "UserSelectionRoleManagedRole",
    "UserSelectionRoleUnmanagedRole",
    "UsersAddRoles",
    "UsersAddRolesAdmin",
    "UsersAddRolesAdminUser",
    "UsersAddRolesAdminUserMutate",
    "UsersAddRolesAdminUserMutateAddRolesInvalidInput",
    "UsersAddRolesAdminUserMutateAddRolesOperationError",
    "UsersAddRolesAdminUserMutateAddRolesUser",
    "UsersCreate",
    "UsersCreateAdmin",
    "UsersCreateAdminUser",
    "UsersCreateAdminUserCreateInvalidInput",
    "UsersCreateAdminUserCreateOperationError",
    "UsersCreateAdminUserCreateUser",
    "UsersDelete",
    "UsersDeleteAdmin",
    "UsersDeleteAdminUser",
    "UsersDeleteAdminUserMutate",
    "UsersDeleteAdminUserMutateDeleteInvalidInput",
    "UsersDeleteAdminUserMutateDeleteOk",
    "UsersDeleteAdminUserMutateDeleteOperationError",
    "UsersGet",
    "UsersGetAdmin",
    "UsersGetAdminUser",
    "UsersGetAdminUserGet",
    "UsersList",
    "UsersListAdmin",
    "UsersListAdminUser",
    "UsersListAdminUserList",
    "UsersRemoveRoles",
    "UsersRemoveRolesAdmin",
    "UsersRemoveRolesAdminUser",
    "UsersRemoveRolesAdminUserMutate",
    "UsersRemoveRolesAdminUserMutateRemoveRolesInvalidInput",
    "UsersRemoveRolesAdminUserMutateRemoveRolesOperationError",
    "UsersRemoveRolesAdminUserMutateRemoveRolesUser",
    "UsersResetPassword",
    "UsersResetPasswordAdmin",
    "UsersResetPasswordAdminUser",
    "UsersResetPasswordAdminUserMutate",
    "UsersResetPasswordAdminUserMutateResetPasswordInvalidInput",
    "UsersResetPasswordAdminUserMutateResetPasswordOk",
    "UsersResetPasswordAdminUserMutateResetPasswordOperationError",
    "UsersSetActive",
    "UsersSetActiveAdmin",
    "UsersSetActiveAdminUser",
    "UsersSetActiveAdminUserMutate",
    "UsersSetActiveAdminUserMutateSetActiveInvalidInput",
    "UsersSetActiveAdminUserMutateSetActiveOperationError",
    "UsersSetActiveAdminUserMutateSetActiveUser",
    "UsersSetAdmin",
    "UsersSetAdminAdmin",
    "UsersSetAdminAdminUser",
    "UsersSetAdminAdminUserMutate",
    "UsersSetAdminAdminUserMutateSetAdminInvalidInput",
    "UsersSetAdminAdminUserMutateSetAdminOperationError",
    "UsersSetAdminAdminUserMutateSetAdminUser",
    "UsersSetEmail",
    "UsersSetEmailAdmin",
    "UsersSetEmailAdminUser",
    "UsersSetEmailAdminUserMutate",
    "UsersSetEmailAdminUserMutateSetEmailInvalidInput",
    "UsersSetEmailAdminUserMutateSetEmailOperationError",
    "UsersSetEmailAdminUserMutateSetEmailUser",
    "UsersSetRole",
    "UsersSetRoleAdmin",
    "UsersSetRoleAdminUser",
    "UsersSetRoleAdminUserMutate",
    "UsersSetRoleAdminUserMutateSetRoleInvalidInput",
    "UsersSetRoleAdminUserMutateSetRoleOperationError",
    "UsersSetRoleAdminUserMutateSetRoleUser",
]



# Generated by ariadne-codegen
# Source: queries.graphql

from typing import Optional

from pydantic import Field

from .base_model import BaseModel
from .fragments import BucketConfigSelection


class BucketGet(BaseModel):
    bucket_config: Optional["BucketGetBucketConfig"] = Field(alias="bucketConfig")


class BucketGetBucketConfig(BucketConfigSelection):
    pass


BucketGet.model_rebuild()



# Generated by ariadne-codegen
# Source: queries.graphql

from typing import Annotated, Literal, Optional, Union

from pydantic import Field

from .base_model import BaseModel
from .fragments import (
    InvalidInputSelection,
    OperationErrorSelection,
    SsoConfigSelection,
)


class SsoConfigSet(BaseModel):
    admin: "SsoConfigSetAdmin"


class SsoConfigSetAdmin(BaseModel):
    set_sso_config: Optional[
        Annotated[
            Union[
                "SsoConfigSetAdminSetSsoConfigSsoConfig",
                "SsoConfigSetAdminSetSsoConfigInvalidInput",
                "SsoConfigSetAdminSetSsoConfigOperationError",
            ],
            Field(discriminator="typename__"),
        ]
    ] = Field(alias="setSsoConfig")


class SsoConfigSetAdminSetSsoConfigSsoConfig(SsoConfigSelection):
    typename__: Literal["SsoConfig"] = Field(alias="__typename")


class SsoConfigSetAdminSetSsoConfigInvalidInput(InvalidInputSelection):
    typename__: Literal["InvalidInput"] = Field(alias="__typename")


class SsoConfigSetAdminSetSsoConfigOperationError(OperationErrorSelection):
    typename__: Literal["OperationError"] = Field(alias="__typename")


SsoConfigSet.model_rebuild()
SsoConfigSetAdmin.model_rebuild()



import typing as T

from . import _graphql_client, exceptions, types, util


def get(name: str) -> T.Optional[types.Bucket]:
    """
    Get a specific bucket configuration from the registry.
    Returns `None` if the bucket does not exist.

    Args:
        name: Name of the bucket to get.
    """
    result = util.get_client().bucket_get(name=name)
    if result is None:
        return None
    return types.Bucket(**result.model_dump())


def list() -> list[types.Bucket]:
    """
    List all bucket configurations in the registry.
    """
    return [types.Bucket(**b.model_dump()) for b in util.get_client().buckets_list()]


def add(
    name: str,
    title: str,
    *,
    description: T.Optional[str] = None,
    icon_url: T.Optional[str] = None,
    overview_url: T.Optional[str] = None,
    tags: T.Optional[T.List[str]] = None,
    relevance_score: T.Optional[int] = None,
    sns_notification_arn: T.Optional[str] = None,
    scanner_parallel_shards_depth: T.Optional[int] = None,
    skip_meta_data_indexing: T.Optional[bool] = None,
    file_extensions_to_index: T.Optional[T.List[str]] = None,
    index_content_bytes: T.Optional[int] = None,
    delay_scan: T.Optional[bool] = None,
    browsable: T.Optional[bool] = None,
    prefixes: T.Optional[T.List[str]] = None,
) -> types.Bucket:
    """
    Add a new bucket to the registry.

    Args:
        name: S3 bucket name.
        title: Display title for the bucket.
        description: Optional description.
        icon_url: Optional URL for bucket icon.
        overview_url: Optional URL for bucket overview page.
        tags: Optional list of tags.
        relevance_score: Optional relevance score for bucket ordering.
        sns_notification_arn: Optional SNS topic ARN for notifications.
        scanner_parallel_shards_depth: Optional depth for parallel scanning.
        skip_meta_data_indexing: If True, skip metadata indexing.
        file_extensions_to_index: Optional list of file extensions to index content.
        index_content_bytes: Optional max bytes of content to index.
        delay_scan: If True, delay initial bucket scan.
        browsable: If True, bucket is browsable.
        prefixes: Optional list of S3 prefixes to scope bucket access to.
            If provided, only these prefixes will be indexed and verified for access.
    """
    result = util.get_client().bucket_add(
        input=_graphql_client.BucketAddInput(
            name=name,
            title=title,
            description=description,
            icon_url=icon_url,
            overview_url=overview_url,
            tags=tags,
            relevance_score=relevance_score,
            sns_notification_arn=sns_notification_arn,
            scanner_parallel_shards_depth=scanner_parallel_shards_depth,
            skip_meta_data_indexing=skip_meta_data_indexing,
            file_extensions_to_index=file_extensions_to_index,
            index_content_bytes=index_content_bytes,
            delay_scan=delay_scan,
            browsable=browsable,
            prefixes=prefixes,
        )
    )
    return _handle_bucket_add_result(result)


def _handle_bucket_add_result(result) -> types.Bucket:
    """Handle bucket add mutation result."""
    if isinstance(result, _graphql_client.BucketAddBucketAddBucketAddSuccess):
        return types.Bucket(**result.bucket_config.model_dump())
    if isinstance(result, _graphql_client.BucketAddBucketAddBucketAlreadyAdded):
        raise exceptions.Quilt3AdminError("Bucket already added")
    if isinstance(result, _graphql_client.BucketAddBucketAddBucketDoesNotExist):
        raise exceptions.Quilt3AdminError("Bucket does not exist in S3")
    if isinstance(result, _graphql_client.BucketAddBucketAddInsufficientPermissions):
        raise exceptions.Quilt3AdminError(result.message)
    if isinstance(result, _graphql_client.BucketAddBucketAddSnsInvalid):
        raise exceptions.Quilt3AdminError("Invalid SNS notification ARN")
    if isinstance(result, _graphql_client.BucketAddBucketAddNotificationConfigurationError):
        raise exceptions.Quilt3AdminError("Notification configuration error")
    if isinstance(result, _graphql_client.BucketAddBucketAddNotificationTopicNotFound):
        raise exceptions.Quilt3AdminError("Notification topic not found")
    if isinstance(result, _graphql_client.BucketAddBucketAddBucketFileExtensionsToIndexInvalid):
        raise exceptions.Quilt3AdminError("Invalid file extensions to index")
    if isinstance(result, _graphql_client.BucketAddBucketAddBucketIndexContentBytesInvalid):
        raise exceptions.Quilt3AdminError("Invalid index content bytes")
    if isinstance(result, _graphql_client.BucketAddBucketAddSubscriptionInvalid):
        raise exceptions.Quilt3AdminError("Invalid subscription")
    raise exceptions.Quilt3AdminError(f"Unknown error: {result}")


def update(
    name: str,
    title: str,
    *,
    description: T.Optional[str] = None,
    icon_url: T.Optional[str] = None,
    overview_url: T.Optional[str] = None,
    tags: T.Optional[T.List[str]] = None,
    relevance_score: T.Optional[int] = None,
    sns_notification_arn: T.Optional[str] = None,
    scanner_parallel_shards_depth: T.Optional[int] = None,
    skip_meta_data_indexing: T.Optional[bool] = None,
    file_extensions_to_index: T.Optional[T.List[str]] = None,
    index_content_bytes: T.Optional[int] = None,
    browsable: T.Optional[bool] = None,
    prefixes: T.Optional[T.List[str]] = None,
) -> types.Bucket:
    """
    Update an existing bucket configuration.

    Args:
        name: S3 bucket name.
        title: Display title for the bucket.
        description: Optional description.
        icon_url: Optional URL for bucket icon.
        overview_url: Optional URL for bucket overview page.
        tags: Optional list of tags.
        relevance_score: Optional relevance score for bucket ordering.
        sns_notification_arn: Optional SNS topic ARN for notifications.
        scanner_parallel_shards_depth: Optional depth for parallel scanning.
        skip_meta_data_indexing: If True, skip metadata indexing.
        file_extensions_to_index: Optional list of file extensions to index content.
        index_content_bytes: Optional max bytes of content to index.
        browsable: If True, bucket is browsable.
        prefixes: Optional list of S3 prefixes to scope bucket access to.
            If provided, only these prefixes will be indexed and verified for access.
            Changing prefixes will trigger permission re-verification.
    """
    result = util.get_client().bucket_update(
        name=name,
        input=_graphql_client.BucketUpdateInput(
            title=title,
            description=description,
            icon_url=icon_url,
            overview_url=overview_url,
            tags=tags,
            relevance_score=relevance_score,
            sns_notification_arn=sns_notification_arn,
            scanner_parallel_shards_depth=scanner_parallel_shards_depth,
            skip_meta_data_indexing=skip_meta_data_indexing,
            file_extensions_to_index=file_extensions_to_index,
            index_content_bytes=index_content_bytes,
            browsable=browsable,
            prefixes=prefixes,
        ),
    )
    return _handle_bucket_update_result(result)


def _handle_bucket_update_result(result) -> types.Bucket:
    """Handle bucket update mutation result."""
    if isinstance(result, _graphql_client.BucketUpdateBucketUpdateBucketUpdateSuccess):
        return types.Bucket(**result.bucket_config.model_dump())
    if isinstance(result, _graphql_client.BucketUpdateBucketUpdateBucketNotFound):
        raise exceptions.BucketNotFoundError()
    if isinstance(result, _graphql_client.BucketUpdateBucketUpdateInsufficientPermissions):
        raise exceptions.Quilt3AdminError(result.message)
    if isinstance(result, _graphql_client.BucketUpdateBucketUpdateSnsInvalid):
        raise exceptions.Quilt3AdminError("Invalid SNS notification ARN")
    if isinstance(result, _graphql_client.BucketUpdateBucketUpdateNotificationConfigurationError):
        raise exceptions.Quilt3AdminError("Notification configuration error")
    if isinstance(result, _graphql_client.BucketUpdateBucketUpdateNotificationTopicNotFound):
        raise exceptions.Quilt3AdminError("Notification topic not found")
    if isinstance(result, _graphql_client.BucketUpdateBucketUpdateBucketFileExtensionsToIndexInvalid):
        raise exceptions.Quilt3AdminError("Invalid file extensions to index")
    if isinstance(result, _graphql_client.BucketUpdateBucketUpdateBucketIndexContentBytesInvalid):
        raise exceptions.Quilt3AdminError("Invalid index content bytes")
    raise exceptions.Quilt3AdminError(f"Unknown error: {result}")


def remove(name: str) -> None:
    """
    Remove a bucket from the registry.

    Args:
        name: Name of the bucket to remove.
    """
    result = util.get_client().bucket_remove(name=name)
    _handle_bucket_remove_result(result)


def _handle_bucket_remove_result(result) -> None:
    """Handle bucket remove mutation result."""
    if isinstance(result, _graphql_client.BucketRemoveBucketRemoveBucketRemoveSuccess):
        return
    if isinstance(result, _graphql_client.BucketRemoveBucketRemoveBucketNotFound):
        raise exceptions.BucketNotFoundError()
    if isinstance(result, _graphql_client.BucketRemoveBucketRemoveIndexingInProgress):
        raise exceptions.Quilt3AdminError("Cannot remove bucket while indexing is in progress")
    raise exceptions.Quilt3AdminError(f"Unknown error: {result}")



import atexit
import functools
import os
import platform
import sys
import uuid
from concurrent.futures import ThreadPoolExecutor, wait
from threading import Lock

from requests_futures.sessions import FuturesSession

from . import __version__ as quilt3_version
from .util import get_from_config, set_config_value

TELEMETRY_URL = "https://telemetry.quiltdata.cloud/Prod/metrics"
TELEMETRY_USER_AGENT = "QuiltCli"
TELEMETRY_CLIENT_TYPE = "quilt3-python-client"
TELEMETRY_SCHEMA_VERSION = "pyclient-usage-metrics-v1"

DISABLE_USAGE_METRICS_ENVVAR = "QUILT_DISABLE_USAGE_METRICS"
MAX_CLEANUP_WAIT_SECS = 5


@functools.cache
def get_session_id():
    return str(uuid.uuid4())


reset_session_id = get_session_id.cache_clear


class ApiTelemetry:
    session = None
    pending_reqs = []
    pending_reqs_lock = Lock()
    telemetry_disabled = None

    @classmethod
    def create_session(cls):
        return FuturesSession(executor=ThreadPoolExecutor(max_workers=2))  # pylint: disable=consider-using-with

    def __init__(self, api_name):
        if ApiTelemetry.telemetry_disabled is None:
            ApiTelemetry.telemetry_disabled = ApiTelemetry.telemetry_is_disabled()

        if ApiTelemetry.session is None:
            ApiTelemetry.session = ApiTelemetry.create_session()

        self.api_name = api_name

    @classmethod
    def has_connectivity(cls):
        # TODO: Implement this check. Punting for the near term
        return True

    @classmethod
    def check_telemetry_disabled_in_config(cls):
        """
        Check if 'telemetry_disabled' field exists in quilt3 config. If it does, return it. If it does not exist, set
        it to default value of 'false' (to handle case of current users who predate this config field).
        """
        config_value = get_from_config("telemetry_disabled")
        if config_value is not None:
            return config_value
        else:
            set_config_value("telemetry_disabled", False)
            return False

    @classmethod
    def check_telemetry_disabled_by_envvar(cls):
        envvar = os.environ.get(DISABLE_USAGE_METRICS_ENVVAR, "")
        if envvar.lower() in ("false", "no", "0"):
            return False
        return bool(envvar)

    @classmethod
    def telemetry_is_disabled(cls):
        disabled_via_config = ApiTelemetry.check_telemetry_disabled_in_config()
        disabled_via_envvar = ApiTelemetry.check_telemetry_disabled_by_envvar()
        disabled_due_to_no_connection = not ApiTelemetry.has_connectivity()

        if disabled_via_config:
            return True

        if disabled_via_envvar:
            return True

        if disabled_due_to_no_connection:
            return True

        return False

    @classmethod
    def cleanup_completed_requests(cls):
        if ApiTelemetry.telemetry_disabled:
            return

        # Take this opportunity to clean up any completed requests so that list never gets too large.
        # Might be better to use a done callback
        # - https://docs.python.org/3/library/concurrent.futures.html#concurrent.futures.Future.add_done_callback
        with ApiTelemetry.pending_reqs_lock:
            ApiTelemetry.pending_reqs = [r for r in ApiTelemetry.pending_reqs if not r.done()]

    @classmethod
    def report_api_use(cls, api_name, python_session_id):
        if ApiTelemetry.telemetry_disabled:
            return

        navigator_url = get_from_config("navigator_url")
        data = {
            "api_name": api_name,
            "python_session_id": python_session_id,
            "telemetry_schema_version": TELEMETRY_SCHEMA_VERSION,
            "navigator_url": navigator_url,
            'client_type': TELEMETRY_CLIENT_TYPE,
            'client_version': quilt3_version,
            'platform': sys.platform,
            'python_implementation': platform.python_implementation(),
            'python_version_major': platform.python_version_tuple()[0],
            'python_version_minor': platform.python_version_tuple()[1],
            'python_version_patch': platform.python_version_tuple()[2],
        }
        # print(f"Sending data: {data}")
        with ApiTelemetry.pending_reqs_lock:
            r = ApiTelemetry.session.post(TELEMETRY_URL, json=data, headers={'User-Agent': TELEMETRY_USER_AGENT})
            ApiTelemetry.pending_reqs.append(r)

    def __call__(self, func):
        @functools.wraps(func)
        def decorated(*args, **kwargs):
            ApiTelemetry.cleanup_completed_requests()
            ApiTelemetry.report_api_use(self.api_name, get_session_id())

            results = func(*args, **kwargs)
            # print(f"{len(ApiTelemetry.pending_reqs)} request(s) pending!")

            if self.api_name == "api.disable_telemetry":
                # Quick hack to disable telemetry immediately after the user calls disable_telemetry()
                # TODO(armand): This string matching is extremely brittle. Fix ASAP
                ApiTelemetry.telemetry_disabled = True
            return results

        return decorated


# Finish up any pending requests, but don't wait forever
@atexit.register
def cleanup():
    # t = Timer("cleanup").start()
    # print(f"Cleaning up {len(ApiTelemetry.pending_reqs)} pending requests...")
    with ApiTelemetry.pending_reqs_lock:  # Not sure why we would need this lock...
        wait(ApiTelemetry.pending_reqs, timeout=MAX_CLEANUP_WAIT_SECS)
    # t.stop()



# Generated by ariadne-codegen
# Source: queries.graphql

from pydantic import Field

from .base_model import BaseModel


class TabulatorGetOpenQuery(BaseModel):
    admin: "TabulatorGetOpenQueryAdmin"


class TabulatorGetOpenQueryAdmin(BaseModel):
    tabulator_open_query: bool = Field(alias="tabulatorOpenQuery")


TabulatorGetOpenQuery.model_rebuild()



# Generated by ariadne-codegen
# Source: queries.graphql

from typing import List

from pydantic import Field

from .base_model import BaseModel
from .fragments import BucketConfigSelection


class BucketsList(BaseModel):
    bucket_configs: List["BucketsListBucketConfigs"] = Field(alias="bucketConfigs")


class BucketsListBucketConfigs(BucketConfigSelection):
    pass


BucketsList.model_rebuild()



from datetime import datetime
from typing import Annotated, List, Literal, Optional, Union

import pydantic


@pydantic.dataclasses.dataclass
class ManagedRole:
    id: str
    name: str
    arn: str
    typename__: Literal["ManagedRole"]


@pydantic.dataclasses.dataclass
class UnmanagedRole:
    id: str
    name: str
    arn: str
    typename__: Literal["UnmanagedRole"]


Role = Union[ManagedRole, UnmanagedRole]
AnnotatedRole = Annotated[Role, pydantic.Field(discriminator="typename__")]
role_adapter = pydantic.TypeAdapter(AnnotatedRole)


@pydantic.dataclasses.dataclass
class User:
    name: str
    email: str
    date_joined: datetime
    last_login: datetime
    is_active: bool
    is_admin: bool
    is_sso_only: bool
    is_service: bool
    role: Optional[AnnotatedRole]
    extra_roles: List[AnnotatedRole]


@pydantic.dataclasses.dataclass
class SSOConfig:
    text: str
    timestamp: datetime
    uploader: User


@pydantic.dataclasses.dataclass
class TabulatorTable:
    name: str
    config: str


@pydantic.dataclasses.dataclass
class Bucket:
    name: str
    title: str
    icon_url: Optional[str]
    description: Optional[str]
    overview_url: Optional[str]
    tags: Optional[List[str]]
    relevance_score: int
    last_indexed: Optional[datetime]
    sns_notification_arn: Optional[str]
    scanner_parallel_shards_depth: Optional[int]
    skip_meta_data_indexing: Optional[bool]
    file_extensions_to_index: Optional[List[str]]
    index_content_bytes: Optional[int]
    prefixes: List[str]



# Generated by ariadne-codegen
# Source: queries.graphql

from typing import Literal, Union

from pydantic import Field

from .base_model import BaseModel
from .fragments import BucketConfigSelection


class BucketUpdate(BaseModel):
    bucket_update: Union[
        "BucketUpdateBucketUpdateBucketUpdateSuccess",
        "BucketUpdateBucketUpdateBucketFileExtensionsToIndexInvalid",
        "BucketUpdateBucketUpdateBucketIndexContentBytesInvalid",
        "BucketUpdateBucketUpdateBucketNotFound",
        "BucketUpdateBucketUpdateInsufficientPermissions",
        "BucketUpdateBucketUpdateNotificationConfigurationError",
        "BucketUpdateBucketUpdateNotificationTopicNotFound",
        "BucketUpdateBucketUpdateSnsInvalid",
    ] = Field(alias="bucketUpdate", discriminator="typename__")


class BucketUpdateBucketUpdateBucketUpdateSuccess(BaseModel):
    typename__: Literal["BucketUpdateSuccess"] = Field(alias="__typename")
    bucket_config: "BucketUpdateBucketUpdateBucketUpdateSuccessBucketConfig" = Field(alias="bucketConfig")


class BucketUpdateBucketUpdateBucketUpdateSuccessBucketConfig(BucketConfigSelection):
    pass


class BucketUpdateBucketUpdateBucketFileExtensionsToIndexInvalid(BaseModel):
    typename__: Literal["BucketFileExtensionsToIndexInvalid"] = Field(alias="__typename")


class BucketUpdateBucketUpdateBucketIndexContentBytesInvalid(BaseModel):
    typename__: Literal["BucketIndexContentBytesInvalid"] = Field(alias="__typename")


class BucketUpdateBucketUpdateBucketNotFound(BaseModel):
    typename__: Literal["BucketNotFound"] = Field(alias="__typename")


class BucketUpdateBucketUpdateInsufficientPermissions(BaseModel):
    typename__: Literal["InsufficientPermissions"] = Field(alias="__typename")
    message: str


class BucketUpdateBucketUpdateNotificationConfigurationError(BaseModel):
    typename__: Literal["NotificationConfigurationError"] = Field(alias="__typename")


class BucketUpdateBucketUpdateNotificationTopicNotFound(BaseModel):
    typename__: Literal["NotificationTopicNotFound"] = Field(alias="__typename")


class BucketUpdateBucketUpdateSnsInvalid(BaseModel):
    typename__: Literal["SnsInvalid"] = Field(alias="__typename")


BucketUpdate.model_rebuild()
BucketUpdateBucketUpdateBucketUpdateSuccess.model_rebuild()



# Generated by ariadne-codegen
# Source: queries.graphql

from typing import Literal, Union

from pydantic import Field

from .base_model import BaseModel
from .fragments import InvalidInputSelection, OperationErrorSelection


class BucketTabulatorTableRename(BaseModel):
    admin: "BucketTabulatorTableRenameAdmin"


class BucketTabulatorTableRenameAdmin(BaseModel):
    bucket_rename_tabulator_table: Union[
        "BucketTabulatorTableRenameAdminBucketRenameTabulatorTableBucketConfig",
        "BucketTabulatorTableRenameAdminBucketRenameTabulatorTableInvalidInput",
        "BucketTabulatorTableRenameAdminBucketRenameTabulatorTableOperationError",
    ] = Field(alias="bucketRenameTabulatorTable", discriminator="typename__")


class BucketTabulatorTableRenameAdminBucketRenameTabulatorTableBucketConfig(BaseModel):
    typename__: Literal["BucketConfig"] = Field(alias="__typename")


class BucketTabulatorTableRenameAdminBucketRenameTabulatorTableInvalidInput(InvalidInputSelection):
    typename__: Literal["InvalidInput"] = Field(alias="__typename")


class BucketTabulatorTableRenameAdminBucketRenameTabulatorTableOperationError(OperationErrorSelection):
    typename__: Literal["OperationError"] = Field(alias="__typename")


BucketTabulatorTableRename.model_rebuild()
BucketTabulatorTableRenameAdmin.model_rebuild()



class Quilt3AdminError(Exception):
    def __init__(self, details):
        super().__init__(details)
        self.details = details


class UserNotFoundError(Quilt3AdminError):
    def __init__(self):
        super().__init__(None)


class BucketNotFoundError(Quilt3AdminError):
    def __init__(self):
        super().__init__(None)



# Generated by ariadne-codegen
# Source: queries.graphql

from datetime import datetime
from typing import Annotated, Any, List, Literal, Optional, Union

from pydantic import Field

from .base_model import BaseModel


class BucketConfigSelection(BaseModel):
    name: str
    title: str
    icon_url: Optional[str] = Field(alias="iconUrl")
    description: Optional[str]
    overview_url: Optional[str] = Field(alias="overviewUrl")
    tags: Optional[List[str]]
    relevance_score: int = Field(alias="relevanceScore")
    last_indexed: Optional[datetime] = Field(alias="lastIndexed")
    sns_notification_arn: Optional[str] = Field(alias="snsNotificationArn")
    scanner_parallel_shards_depth: Optional[int] = Field(alias="scannerParallelShardsDepth")
    skip_meta_data_indexing: Optional[bool] = Field(alias="skipMetaDataIndexing")
    file_extensions_to_index: Optional[List[str]] = Field(alias="fileExtensionsToIndex")
    index_content_bytes: Optional[int] = Field(alias="indexContentBytes")
    prefixes: List[str]


class InvalidInputSelection(BaseModel):
    errors: List["InvalidInputSelectionErrors"]


class InvalidInputSelectionErrors(BaseModel):
    path: Optional[str]
    message: str
    name: str
    context: Optional[Any]


class ManagedRoleSelection(BaseModel):
    id: str
    name: str
    arn: str


class OperationErrorSelection(BaseModel):
    message: str
    name: str
    context: Optional[Any]


class UnmanagedRoleSelection(BaseModel):
    id: str
    name: str
    arn: str


class UserSelection(BaseModel):
    name: str
    email: str
    date_joined: datetime = Field(alias="dateJoined")
    last_login: datetime = Field(alias="lastLogin")
    is_active: bool = Field(alias="isActive")
    is_admin: bool = Field(alias="isAdmin")
    is_sso_only: bool = Field(alias="isSsoOnly")
    is_service: bool = Field(alias="isService")
    role: Optional[
        Annotated[
            Union["UserSelectionRoleUnmanagedRole", "UserSelectionRoleManagedRole"],
            Field(discriminator="typename__"),
        ]
    ]
    extra_roles: List[
        Annotated[
            Union[
                "UserSelectionExtraRolesUnmanagedRole",
                "UserSelectionExtraRolesManagedRole",
            ],
            Field(discriminator="typename__"),
        ]
    ] = Field(alias="extraRoles")


class UserSelectionRoleUnmanagedRole(UnmanagedRoleSelection):
    typename__: Literal["UnmanagedRole"] = Field(alias="__typename")


class UserSelectionRoleManagedRole(ManagedRoleSelection):
    typename__: Literal["ManagedRole"] = Field(alias="__typename")


class UserSelectionExtraRolesUnmanagedRole(UnmanagedRoleSelection):
    typename__: Literal["UnmanagedRole"] = Field(alias="__typename")


class UserSelectionExtraRolesManagedRole(ManagedRoleSelection):
    typename__: Literal["ManagedRole"] = Field(alias="__typename")


class SsoConfigSelection(BaseModel):
    text: str
    timestamp: datetime
    uploader: "SsoConfigSelectionUploader"


class SsoConfigSelectionUploader(UserSelection):
    pass


BucketConfigSelection.model_rebuild()
InvalidInputSelection.model_rebuild()
ManagedRoleSelection.model_rebuild()
OperationErrorSelection.model_rebuild()
UnmanagedRoleSelection.model_rebuild()
UserSelection.model_rebuild()
SsoConfigSelection.model_rebuild()



# Generated by ariadne-codegen
# Source: queries.graphql

from typing import Literal, Union

from pydantic import Field

from .base_model import BaseModel


class BucketRemove(BaseModel):
    bucket_remove: Union[
        "BucketRemoveBucketRemoveBucketRemoveSuccess",
        "BucketRemoveBucketRemoveBucketNotFound",
        "BucketRemoveBucketRemoveIndexingInProgress",
    ] = Field(alias="bucketRemove", discriminator="typename__")


class BucketRemoveBucketRemoveBucketRemoveSuccess(BaseModel):
    typename__: Literal["BucketRemoveSuccess"] = Field(alias="__typename")


class BucketRemoveBucketRemoveBucketNotFound(BaseModel):
    typename__: Literal["BucketNotFound"] = Field(alias="__typename")


class BucketRemoveBucketRemoveIndexingInProgress(BaseModel):
    typename__: Literal["IndexingInProgress"] = Field(alias="__typename")


BucketRemove.model_rebuild()



# Generated by ariadne-codegen
# Source: queries.graphql

from pydantic import Field

from .base_model import BaseModel


class TabulatorSetOpenQuery(BaseModel):
    admin: "TabulatorSetOpenQueryAdmin"


class TabulatorSetOpenQueryAdmin(BaseModel):
    set_tabulator_open_query: "TabulatorSetOpenQueryAdminSetTabulatorOpenQuery" = Field(alias="setTabulatorOpenQuery")


class TabulatorSetOpenQueryAdminSetTabulatorOpenQuery(BaseModel):
    tabulator_open_query: bool = Field(alias="tabulatorOpenQuery")


TabulatorSetOpenQuery.model_rebuild()
TabulatorSetOpenQueryAdmin.model_rebuild()



# Generated by ariadne-codegen
# Source: queries.graphql

from typing import Any, Dict, List, Optional, Union

from .base_client import BaseClient
from .base_model import UNSET, UnsetType
from .bucket_add import (
    BucketAdd,
    BucketAddBucketAddBucketAddSuccess,
    BucketAddBucketAddBucketAlreadyAdded,
    BucketAddBucketAddBucketDoesNotExist,
    BucketAddBucketAddBucketFileExtensionsToIndexInvalid,
    BucketAddBucketAddBucketIndexContentBytesInvalid,
    BucketAddBucketAddInsufficientPermissions,
    BucketAddBucketAddNotificationConfigurationError,
    BucketAddBucketAddNotificationTopicNotFound,
    BucketAddBucketAddSnsInvalid,
    BucketAddBucketAddSubscriptionInvalid,
)
from .bucket_get import BucketGet, BucketGetBucketConfig
from .bucket_remove import (
    BucketRemove,
    BucketRemoveBucketRemoveBucketNotFound,
    BucketRemoveBucketRemoveBucketRemoveSuccess,
    BucketRemoveBucketRemoveIndexingInProgress,
)
from .bucket_tabulator_table_rename import (
    BucketTabulatorTableRename,
    BucketTabulatorTableRenameAdminBucketRenameTabulatorTableBucketConfig,
    BucketTabulatorTableRenameAdminBucketRenameTabulatorTableInvalidInput,
    BucketTabulatorTableRenameAdminBucketRenameTabulatorTableOperationError,
)
from .bucket_tabulator_table_set import (
    BucketTabulatorTableSet,
    BucketTabulatorTableSetAdminBucketSetTabulatorTableBucketConfig,
    BucketTabulatorTableSetAdminBucketSetTabulatorTableInvalidInput,
    BucketTabulatorTableSetAdminBucketSetTabulatorTableOperationError,
)
from .bucket_tabulator_tables_list import (
    BucketTabulatorTablesList,
    BucketTabulatorTablesListBucketConfig,
)
from .bucket_update import (
    BucketUpdate,
    BucketUpdateBucketUpdateBucketFileExtensionsToIndexInvalid,
    BucketUpdateBucketUpdateBucketIndexContentBytesInvalid,
    BucketUpdateBucketUpdateBucketNotFound,
    BucketUpdateBucketUpdateBucketUpdateSuccess,
    BucketUpdateBucketUpdateInsufficientPermissions,
    BucketUpdateBucketUpdateNotificationConfigurationError,
    BucketUpdateBucketUpdateNotificationTopicNotFound,
    BucketUpdateBucketUpdateSnsInvalid,
)
from .buckets_list import BucketsList, BucketsListBucketConfigs
from .input_types import BucketAddInput, BucketUpdateInput, UserInput
from .roles_list import (
    RolesList,
    RolesListRolesManagedRole,
    RolesListRolesUnmanagedRole,
)
from .sso_config_get import SsoConfigGet, SsoConfigGetAdminSsoConfig
from .sso_config_set import (
    SsoConfigSet,
    SsoConfigSetAdminSetSsoConfigInvalidInput,
    SsoConfigSetAdminSetSsoConfigOperationError,
    SsoConfigSetAdminSetSsoConfigSsoConfig,
)
from .tabulator_get_open_query import TabulatorGetOpenQuery
from .tabulator_set_open_query import TabulatorSetOpenQuery
from .users_add_roles import UsersAddRoles, UsersAddRolesAdminUserMutate
from .users_create import (
    UsersCreate,
    UsersCreateAdminUserCreateInvalidInput,
    UsersCreateAdminUserCreateOperationError,
    UsersCreateAdminUserCreateUser,
)
from .users_delete import UsersDelete, UsersDeleteAdminUserMutate
from .users_get import UsersGet, UsersGetAdminUserGet
from .users_list import UsersList, UsersListAdminUserList
from .users_remove_roles import UsersRemoveRoles, UsersRemoveRolesAdminUserMutate
from .users_reset_password import UsersResetPassword, UsersResetPasswordAdminUserMutate
from .users_set_active import UsersSetActive, UsersSetActiveAdminUserMutate
from .users_set_admin import UsersSetAdmin, UsersSetAdminAdminUserMutate
from .users_set_email import UsersSetEmail, UsersSetEmailAdminUserMutate
from .users_set_role import UsersSetRole, UsersSetRoleAdminUserMutate


def gql(q: str) -> str:
    return q


class Client(BaseClient):
    def roles_list(self, **kwargs: Any) -> List[Union[RolesListRolesUnmanagedRole, RolesListRolesManagedRole]]:
        query = gql(
            """
            query rolesList {
              roles {
                ...RoleSelection
              }
            }

            fragment ManagedRoleSelection on ManagedRole {
              id
              name
              arn
            }

            fragment RoleSelection on Role {
              __typename
              ...UnmanagedRoleSelection
              ...ManagedRoleSelection
            }

            fragment UnmanagedRoleSelection on UnmanagedRole {
              id
              name
              arn
            }
            """
        )
        variables: Dict[str, object] = {}
        response = self.execute(query=query, operation_name="rolesList", variables=variables, **kwargs)
        data = self.get_data(response)
        return RolesList.model_validate(data).roles

    def users_get(self, name: str, **kwargs: Any) -> Optional[UsersGetAdminUserGet]:
        query = gql(
            """
            query usersGet($name: String!) {
              admin {
                user {
                  get(name: $name) {
                    ...UserSelection
                  }
                }
              }
            }

            fragment ManagedRoleSelection on ManagedRole {
              id
              name
              arn
            }

            fragment RoleSelection on Role {
              __typename
              ...UnmanagedRoleSelection
              ...ManagedRoleSelection
            }

            fragment UnmanagedRoleSelection on UnmanagedRole {
              id
              name
              arn
            }

            fragment UserSelection on User {
              name
              email
              dateJoined
              lastLogin
              isActive
              isAdmin
              isSsoOnly
              isService
              role {
                ...RoleSelection
              }
              extraRoles {
                ...RoleSelection
              }
            }
            """
        )
        variables: Dict[str, object] = {"name": name}
        response = self.execute(query=query, operation_name="usersGet", variables=variables, **kwargs)
        data = self.get_data(response)
        return UsersGet.model_validate(data).admin.user.get

    def users_list(self, **kwargs: Any) -> List[UsersListAdminUserList]:
        query = gql(
            """
            query usersList {
              admin {
                user {
                  list {
                    ...UserSelection
                  }
                }
              }
            }

            fragment ManagedRoleSelection on ManagedRole {
              id
              name
              arn
            }

            fragment RoleSelection on Role {
              __typename
              ...UnmanagedRoleSelection
              ...ManagedRoleSelection
            }

            fragment UnmanagedRoleSelection on UnmanagedRole {
              id
              name
              arn
            }

            fragment UserSelection on User {
              name
              email
              dateJoined
              lastLogin
              isActive
              isAdmin
              isSsoOnly
              isService
              role {
                ...RoleSelection
              }
              extraRoles {
                ...RoleSelection
              }
            }
            """
        )
        variables: Dict[str, object] = {}
        response = self.execute(query=query, operation_name="usersList", variables=variables, **kwargs)
        data = self.get_data(response)
        return UsersList.model_validate(data).admin.user.list

    def users_create(
        self, input: UserInput, **kwargs: Any
    ) -> Union[
        UsersCreateAdminUserCreateUser,
        UsersCreateAdminUserCreateInvalidInput,
        UsersCreateAdminUserCreateOperationError,
    ]:
        query = gql(
            """
            mutation usersCreate($input: UserInput!) {
              admin {
                user {
                  create(input: $input) {
                    __typename
                    ...UserMutationSelection
                  }
                }
              }
            }

            fragment InvalidInputSelection on InvalidInput {
              errors {
                path
                message
                name
                context
              }
            }

            fragment ManagedRoleSelection on ManagedRole {
              id
              name
              arn
            }

            fragment OperationErrorSelection on OperationError {
              message
              name
              context
            }

            fragment RoleSelection on Role {
              __typename
              ...UnmanagedRoleSelection
              ...ManagedRoleSelection
            }

            fragment UnmanagedRoleSelection on UnmanagedRole {
              id
              name
              arn
            }

            fragment UserMutationSelection on UserResult {
              ...UserSelection
              ...InvalidInputSelection
              ...OperationErrorSelection
            }

            fragment UserSelection on User {
              name
              email
              dateJoined
              lastLogin
              isActive
              isAdmin
              isSsoOnly
              isService
              role {
                ...RoleSelection
              }
              extraRoles {
                ...RoleSelection
              }
            }
            """
        )
        variables: Dict[str, object] = {"input": input}
        response = self.execute(query=query, operation_name="usersCreate", variables=variables, **kwargs)
        data = self.get_data(response)
        return UsersCreate.model_validate(data).admin.user.create

    def users_delete(self, name: str, **kwargs: Any) -> Optional[UsersDeleteAdminUserMutate]:
        query = gql(
            """
            mutation usersDelete($name: String!) {
              admin {
                user {
                  mutate(name: $name) {
                    delete {
                      __typename
                      ...InvalidInputSelection
                      ...OperationErrorSelection
                    }
                  }
                }
              }
            }

            fragment InvalidInputSelection on InvalidInput {
              errors {
                path
                message
                name
                context
              }
            }

            fragment OperationErrorSelection on OperationError {
              message
              name
              context
            }
            """
        )
        variables: Dict[str, object] = {"name": name}
        response = self.execute(query=query, operation_name="usersDelete", variables=variables, **kwargs)
        data = self.get_data(response)
        return UsersDelete.model_validate(data).admin.user.mutate

    def users_set_email(self, email: str, name: str, **kwargs: Any) -> Optional[UsersSetEmailAdminUserMutate]:
        query = gql(
            """
            mutation usersSetEmail($email: String!, $name: String!) {
              admin {
                user {
                  mutate(name: $name) {
                    setEmail(email: $email) {
                      __typename
                      ...UserMutationSelection
                    }
                  }
                }
              }
            }

            fragment InvalidInputSelection on InvalidInput {
              errors {
                path
                message
                name
                context
              }
            }

            fragment ManagedRoleSelection on ManagedRole {
              id
              name
              arn
            }

            fragment OperationErrorSelection on OperationError {
              message
              name
              context
            }

            fragment RoleSelection on Role {
              __typename
              ...UnmanagedRoleSelection
              ...ManagedRoleSelection
            }

            fragment UnmanagedRoleSelection on UnmanagedRole {
              id
              name
              arn
            }

            fragment UserMutationSelection on UserResult {
              ...UserSelection
              ...InvalidInputSelection
              ...OperationErrorSelection
            }

            fragment UserSelection on User {
              name
              email
              dateJoined
              lastLogin
              isActive
              isAdmin
              isSsoOnly
              isService
              role {
                ...RoleSelection
              }
              extraRoles {
                ...RoleSelection
              }
            }
            """
        )
        variables: Dict[str, object] = {"email": email, "name": name}
        response = self.execute(query=query, operation_name="usersSetEmail", variables=variables, **kwargs)
        data = self.get_data(response)
        return UsersSetEmail.model_validate(data).admin.user.mutate

    def users_set_admin(self, name: str, admin: bool, **kwargs: Any) -> Optional[UsersSetAdminAdminUserMutate]:
        query = gql(
            """
            mutation usersSetAdmin($name: String!, $admin: Boolean!) {
              admin {
                user {
                  mutate(name: $name) {
                    setAdmin(admin: $admin) {
                      __typename
                      ...UserMutationSelection
                    }
                  }
                }
              }
            }

            fragment InvalidInputSelection on InvalidInput {
              errors {
                path
                message
                name
                context
              }
            }

            fragment ManagedRoleSelection on ManagedRole {
              id
              name
              arn
            }

            fragment OperationErrorSelection on OperationError {
              message
              name
              context
            }

            fragment RoleSelection on Role {
              __typename
              ...UnmanagedRoleSelection
              ...ManagedRoleSelection
            }

            fragment UnmanagedRoleSelection on UnmanagedRole {
              id
              name
              arn
            }

            fragment UserMutationSelection on UserResult {
              ...UserSelection
              ...InvalidInputSelection
              ...OperationErrorSelection
            }

            fragment UserSelection on User {
              name
              email
              dateJoined
              lastLogin
              isActive
              isAdmin
              isSsoOnly
              isService
              role {
                ...RoleSelection
              }
              extraRoles {
                ...RoleSelection
              }
            }
            """
        )
        variables: Dict[str, object] = {"name": name, "admin": admin}
        response = self.execute(query=query, operation_name="usersSetAdmin", variables=variables, **kwargs)
        data = self.get_data(response)
        return UsersSetAdmin.model_validate(data).admin.user.mutate

    def users_set_active(self, active: bool, name: str, **kwargs: Any) -> Optional[UsersSetActiveAdminUserMutate]:
        query = gql(
            """
            mutation usersSetActive($active: Boolean!, $name: String!) {
              admin {
                user {
                  mutate(name: $name) {
                    setActive(active: $active) {
                      __typename
                      ...UserMutationSelection
                    }
                  }
                }
              }
            }

            fragment InvalidInputSelection on InvalidInput {
              errors {
                path
                message
                name
                context
              }
            }

            fragment ManagedRoleSelection on ManagedRole {
              id
              name
              arn
            }

            fragment OperationErrorSelection on OperationError {
              message
              name
              context
            }

            fragment RoleSelection on Role {
              __typename
              ...UnmanagedRoleSelection
              ...ManagedRoleSelection
            }

            fragment UnmanagedRoleSelection on UnmanagedRole {
              id
              name
              arn
            }

            fragment UserMutationSelection on UserResult {
              ...UserSelection
              ...InvalidInputSelection
              ...OperationErrorSelection
            }

            fragment UserSelection on User {
              name
              email
              dateJoined
              lastLogin
              isActive
              isAdmin
              isSsoOnly
              isService
              role {
                ...RoleSelection
              }
              extraRoles {
                ...RoleSelection
              }
            }
            """
        )
        variables: Dict[str, object] = {"active": active, "name": name}
        response = self.execute(query=query, operation_name="usersSetActive", variables=variables, **kwargs)
        data = self.get_data(response)
        return UsersSetActive.model_validate(data).admin.user.mutate

    def users_reset_password(self, name: str, **kwargs: Any) -> Optional[UsersResetPasswordAdminUserMutate]:
        query = gql(
            """
            mutation usersResetPassword($name: String!) {
              admin {
                user {
                  mutate(name: $name) {
                    resetPassword {
                      __typename
                      ...InvalidInputSelection
                      ...OperationErrorSelection
                    }
                  }
                }
              }
            }

            fragment InvalidInputSelection on InvalidInput {
              errors {
                path
                message
                name
                context
              }
            }

            fragment OperationErrorSelection on OperationError {
              message
              name
              context
            }
            """
        )
        variables: Dict[str, object] = {"name": name}
        response = self.execute(query=query, operation_name="usersResetPassword", variables=variables, **kwargs)
        data = self.get_data(response)
        return UsersResetPassword.model_validate(data).admin.user.mutate

    def users_set_role(
        self,
        name: str,
        role: str,
        append: bool,
        extra_roles: Union[Optional[List[str]], UnsetType] = UNSET,
        **kwargs: Any,
    ) -> Optional[UsersSetRoleAdminUserMutate]:
        query = gql(
            """
            mutation usersSetRole($name: String!, $role: String!, $extraRoles: [String!], $append: Boolean!) {
              admin {
                user {
                  mutate(name: $name) {
                    setRole(role: $role, extraRoles: $extraRoles, append: $append) {
                      __typename
                      ...UserSelection
                      ...InvalidInputSelection
                      ...OperationErrorSelection
                    }
                  }
                }
              }
            }

            fragment InvalidInputSelection on InvalidInput {
              errors {
                path
                message
                name
                context
              }
            }

            fragment ManagedRoleSelection on ManagedRole {
              id
              name
              arn
            }

            fragment OperationErrorSelection on OperationError {
              message
              name
              context
            }

            fragment RoleSelection on Role {
              __typename
              ...UnmanagedRoleSelection
              ...ManagedRoleSelection
            }

            fragment UnmanagedRoleSelection on UnmanagedRole {
              id
              name
              arn
            }

            fragment UserSelection on User {
              name
              email
              dateJoined
              lastLogin
              isActive
              isAdmin
              isSsoOnly
              isService
              role {
                ...RoleSelection
              }
              extraRoles {
                ...RoleSelection
              }
            }
            """
        )
        variables: Dict[str, object] = {
            "name": name,
            "role": role,
            "extraRoles": extra_roles,
            "append": append,
        }
        response = self.execute(query=query, operation_name="usersSetRole", variables=variables, **kwargs)
        data = self.get_data(response)
        return UsersSetRole.model_validate(data).admin.user.mutate

    def users_add_roles(self, name: str, roles: List[str], **kwargs: Any) -> Optional[UsersAddRolesAdminUserMutate]:
        query = gql(
            """
            mutation usersAddRoles($name: String!, $roles: [String!]!) {
              admin {
                user {
                  mutate(name: $name) {
                    addRoles(roles: $roles) {
                      __typename
                      ...UserSelection
                      ...InvalidInputSelection
                      ...OperationErrorSelection
                    }
                  }
                }
              }
            }

            fragment InvalidInputSelection on InvalidInput {
              errors {
                path
                message
                name
                context
              }
            }

            fragment ManagedRoleSelection on ManagedRole {
              id
              name
              arn
            }

            fragment OperationErrorSelection on OperationError {
              message
              name
              context
            }

            fragment RoleSelection on Role {
              __typename
              ...UnmanagedRoleSelection
              ...ManagedRoleSelection
            }

            fragment UnmanagedRoleSelection on UnmanagedRole {
              id
              name
              arn
            }

            fragment UserSelection on User {
              name
              email
              dateJoined
              lastLogin
              isActive
              isAdmin
              isSsoOnly
              isService
              role {
                ...RoleSelection
              }
              extraRoles {
                ...RoleSelection
              }
            }
            """
        )
        variables: Dict[str, object] = {"name": name, "roles": roles}
        response = self.execute(query=query, operation_name="usersAddRoles", variables=variables, **kwargs)
        data = self.get_data(response)
        return UsersAddRoles.model_validate(data).admin.user.mutate

    def users_remove_roles(
        self, name: str, roles: List[str], fallback: Union[Optional[str], UnsetType] = UNSET, **kwargs: Any
    ) -> Optional[UsersRemoveRolesAdminUserMutate]:
        query = gql(
            """
            mutation usersRemoveRoles($name: String!, $roles: [String!]!, $fallback: String) {
              admin {
                user {
                  mutate(name: $name) {
                    removeRoles(roles: $roles, fallback: $fallback) {
                      __typename
                      ...UserSelection
                      ...InvalidInputSelection
                      ...OperationErrorSelection
                    }
                  }
                }
              }
            }

            fragment InvalidInputSelection on InvalidInput {
              errors {
                path
                message
                name
                context
              }
            }

            fragment ManagedRoleSelection on ManagedRole {
              id
              name
              arn
            }

            fragment OperationErrorSelection on OperationError {
              message
              name
              context
            }

            fragment RoleSelection on Role {
              __typename
              ...UnmanagedRoleSelection
              ...ManagedRoleSelection
            }

            fragment UnmanagedRoleSelection on UnmanagedRole {
              id
              name
              arn
            }

            fragment UserSelection on User {
              name
              email
              dateJoined
              lastLogin
              isActive
              isAdmin
              isSsoOnly
              isService
              role {
                ...RoleSelection
              }
              extraRoles {
                ...RoleSelection
              }
            }
            """
        )
        variables: Dict[str, object] = {
            "name": name,
            "roles": roles,
            "fallback": fallback,
        }
        response = self.execute(query=query, operation_name="usersRemoveRoles", variables=variables, **kwargs)
        data = self.get_data(response)
        return UsersRemoveRoles.model_validate(data).admin.user.mutate

    def sso_config_get(self, **kwargs: Any) -> Optional[SsoConfigGetAdminSsoConfig]:
        query = gql(
            """
            query ssoConfigGet {
              admin {
                ssoConfig {
                  ...SsoConfigSelection
                }
              }
            }

            fragment ManagedRoleSelection on ManagedRole {
              id
              name
              arn
            }

            fragment RoleSelection on Role {
              __typename
              ...UnmanagedRoleSelection
              ...ManagedRoleSelection
            }

            fragment SsoConfigSelection on SsoConfig {
              text
              timestamp
              uploader {
                ...UserSelection
              }
            }

            fragment UnmanagedRoleSelection on UnmanagedRole {
              id
              name
              arn
            }

            fragment UserSelection on User {
              name
              email
              dateJoined
              lastLogin
              isActive
              isAdmin
              isSsoOnly
              isService
              role {
                ...RoleSelection
              }
              extraRoles {
                ...RoleSelection
              }
            }
            """
        )
        variables: Dict[str, object] = {}
        response = self.execute(query=query, operation_name="ssoConfigGet", variables=variables, **kwargs)
        data = self.get_data(response)
        return SsoConfigGet.model_validate(data).admin.sso_config

    def sso_config_set(
        self, config: Union[Optional[str], UnsetType] = UNSET, **kwargs: Any
    ) -> Optional[
        Union[
            SsoConfigSetAdminSetSsoConfigSsoConfig,
            SsoConfigSetAdminSetSsoConfigInvalidInput,
            SsoConfigSetAdminSetSsoConfigOperationError,
        ]
    ]:
        query = gql(
            """
            mutation ssoConfigSet($config: String) {
              admin {
                setSsoConfig(config: $config) {
                  __typename
                  ...SsoConfigSelection
                  ...InvalidInputSelection
                  ...OperationErrorSelection
                }
              }
            }

            fragment InvalidInputSelection on InvalidInput {
              errors {
                path
                message
                name
                context
              }
            }

            fragment ManagedRoleSelection on ManagedRole {
              id
              name
              arn
            }

            fragment OperationErrorSelection on OperationError {
              message
              name
              context
            }

            fragment RoleSelection on Role {
              __typename
              ...UnmanagedRoleSelection
              ...ManagedRoleSelection
            }

            fragment SsoConfigSelection on SsoConfig {
              text
              timestamp
              uploader {
                ...UserSelection
              }
            }

            fragment UnmanagedRoleSelection on UnmanagedRole {
              id
              name
              arn
            }

            fragment UserSelection on User {
              name
              email
              dateJoined
              lastLogin
              isActive
              isAdmin
              isSsoOnly
              isService
              role {
                ...RoleSelection
              }
              extraRoles {
                ...RoleSelection
              }
            }
            """
        )
        variables: Dict[str, object] = {"config": config}
        response = self.execute(query=query, operation_name="ssoConfigSet", variables=variables, **kwargs)
        data = self.get_data(response)
        return SsoConfigSet.model_validate(data).admin.set_sso_config

    def bucket_tabulator_tables_list(
        self, name: str, **kwargs: Any
    ) -> Optional[BucketTabulatorTablesListBucketConfig]:
        query = gql(
            """
            query bucketTabulatorTablesList($name: String!) {
              bucketConfig(name: $name) {
                tabulatorTables {
                  name
                  config
                }
              }
            }
            """
        )
        variables: Dict[str, object] = {"name": name}
        response = self.execute(query=query, operation_name="bucketTabulatorTablesList", variables=variables, **kwargs)
        data = self.get_data(response)
        return BucketTabulatorTablesList.model_validate(data).bucket_config

    def bucket_tabulator_table_set(
        self, bucket_name: str, table_name: str, config: Union[Optional[str], UnsetType] = UNSET, **kwargs: Any
    ) -> Union[
        BucketTabulatorTableSetAdminBucketSetTabulatorTableBucketConfig,
        BucketTabulatorTableSetAdminBucketSetTabulatorTableInvalidInput,
        BucketTabulatorTableSetAdminBucketSetTabulatorTableOperationError,
    ]:
        query = gql(
            """
            mutation bucketTabulatorTableSet($bucketName: String!, $tableName: String!, $config: String) {
              admin {
                bucketSetTabulatorTable(
                  bucketName: $bucketName
                  tableName: $tableName
                  config: $config
                ) {
                  __typename
                  ...InvalidInputSelection
                  ...OperationErrorSelection
                }
              }
            }

            fragment InvalidInputSelection on InvalidInput {
              errors {
                path
                message
                name
                context
              }
            }

            fragment OperationErrorSelection on OperationError {
              message
              name
              context
            }
            """
        )
        variables: Dict[str, object] = {
            "bucketName": bucket_name,
            "tableName": table_name,
            "config": config,
        }
        response = self.execute(query=query, operation_name="bucketTabulatorTableSet", variables=variables, **kwargs)
        data = self.get_data(response)
        return BucketTabulatorTableSet.model_validate(data).admin.bucket_set_tabulator_table

    def bucket_tabulator_table_rename(
        self, bucket_name: str, table_name: str, new_table_name: str, **kwargs: Any
    ) -> Union[
        BucketTabulatorTableRenameAdminBucketRenameTabulatorTableBucketConfig,
        BucketTabulatorTableRenameAdminBucketRenameTabulatorTableInvalidInput,
        BucketTabulatorTableRenameAdminBucketRenameTabulatorTableOperationError,
    ]:
        query = gql(
            """
            mutation bucketTabulatorTableRename($bucketName: String!, $tableName: String!, $newTableName: String!) {
              admin {
                bucketRenameTabulatorTable(
                  bucketName: $bucketName
                  tableName: $tableName
                  newTableName: $newTableName
                ) {
                  __typename
                  ...InvalidInputSelection
                  ...OperationErrorSelection
                }
              }
            }

            fragment InvalidInputSelection on InvalidInput {
              errors {
                path
                message
                name
                context
              }
            }

            fragment OperationErrorSelection on OperationError {
              message
              name
              context
            }
            """
        )
        variables: Dict[str, object] = {
            "bucketName": bucket_name,
            "tableName": table_name,
            "newTableName": new_table_name,
        }
        response = self.execute(
            query=query, operation_name="bucketTabulatorTableRename", variables=variables, **kwargs
        )
        data = self.get_data(response)
        return BucketTabulatorTableRename.model_validate(data).admin.bucket_rename_tabulator_table

    def tabulator_get_open_query(self, **kwargs: Any) -> bool:
        query = gql(
            """
            query tabulatorGetOpenQuery {
              admin {
                tabulatorOpenQuery
              }
            }
            """
        )
        variables: Dict[str, object] = {}
        response = self.execute(query=query, operation_name="tabulatorGetOpenQuery", variables=variables, **kwargs)
        data = self.get_data(response)
        return TabulatorGetOpenQuery.model_validate(data).admin.tabulator_open_query

    def tabulator_set_open_query(self, enabled: bool, **kwargs: Any) -> bool:
        query = gql(
            """
            mutation tabulatorSetOpenQuery($enabled: Boolean!) {
              admin {
                setTabulatorOpenQuery(enabled: $enabled) {
                  tabulatorOpenQuery
                }
              }
            }
            """
        )
        variables: Dict[str, object] = {"enabled": enabled}
        response = self.execute(query=query, operation_name="tabulatorSetOpenQuery", variables=variables, **kwargs)
        data = self.get_data(response)
        return TabulatorSetOpenQuery.model_validate(data).admin.set_tabulator_open_query.tabulator_open_query

    def bucket_get(self, name: str, **kwargs: Any) -> Optional[BucketGetBucketConfig]:
        query = gql(
            """
            query bucketGet($name: String!) {
              bucketConfig(name: $name) {
                ...BucketConfigSelection
              }
            }

            fragment BucketConfigSelection on BucketConfig {
              name
              title
              iconUrl
              description
              overviewUrl
              tags
              relevanceScore
              lastIndexed
              snsNotificationArn
              scannerParallelShardsDepth
              skipMetaDataIndexing
              fileExtensionsToIndex
              indexContentBytes
              prefixes
            }
            """
        )
        variables: Dict[str, object] = {"name": name}
        response = self.execute(query=query, operation_name="bucketGet", variables=variables, **kwargs)
        data = self.get_data(response)
        return BucketGet.model_validate(data).bucket_config

    def buckets_list(self, **kwargs: Any) -> List[BucketsListBucketConfigs]:
        query = gql(
            """
            query bucketsList {
              bucketConfigs {
                ...BucketConfigSelection
              }
            }

            fragment BucketConfigSelection on BucketConfig {
              name
              title
              iconUrl
              description
              overviewUrl
              tags
              relevanceScore
              lastIndexed
              snsNotificationArn
              scannerParallelShardsDepth
              skipMetaDataIndexing
              fileExtensionsToIndex
              indexContentBytes
              prefixes
            }
            """
        )
        variables: Dict[str, object] = {}
        response = self.execute(query=query, operation_name="bucketsList", variables=variables, **kwargs)
        data = self.get_data(response)
        return BucketsList.model_validate(data).bucket_configs

    def bucket_add(
        self, input: BucketAddInput, **kwargs: Any
    ) -> Union[
        BucketAddBucketAddBucketAddSuccess,
        BucketAddBucketAddBucketAlreadyAdded,
        BucketAddBucketAddBucketDoesNotExist,
        BucketAddBucketAddBucketFileExtensionsToIndexInvalid,
        BucketAddBucketAddBucketIndexContentBytesInvalid,
        BucketAddBucketAddInsufficientPermissions,
        BucketAddBucketAddNotificationConfigurationError,
        BucketAddBucketAddNotificationTopicNotFound,
        BucketAddBucketAddSnsInvalid,
        BucketAddBucketAddSubscriptionInvalid,
    ]:
        query = gql(
            """
            mutation bucketAdd($input: BucketAddInput!) {
              bucketAdd(input: $input) {
                ...BucketAddResultSelection
              }
            }

            fragment BucketAddResultSelection on BucketAddResult {
              __typename
              ... on BucketAddSuccess {
                bucketConfig {
                  ...BucketConfigSelection
                }
              }
              ... on InsufficientPermissions {
                message
              }
            }

            fragment BucketConfigSelection on BucketConfig {
              name
              title
              iconUrl
              description
              overviewUrl
              tags
              relevanceScore
              lastIndexed
              snsNotificationArn
              scannerParallelShardsDepth
              skipMetaDataIndexing
              fileExtensionsToIndex
              indexContentBytes
              prefixes
            }
            """
        )
        variables: Dict[str, object] = {"input": input}
        response = self.execute(query=query, operation_name="bucketAdd", variables=variables, **kwargs)
        data = self.get_data(response)
        return BucketAdd.model_validate(data).bucket_add

    def bucket_update(
        self, name: str, input: BucketUpdateInput, **kwargs: Any
    ) -> Union[
        BucketUpdateBucketUpdateBucketUpdateSuccess,
        BucketUpdateBucketUpdateBucketFileExtensionsToIndexInvalid,
        BucketUpdateBucketUpdateBucketIndexContentBytesInvalid,
        BucketUpdateBucketUpdateBucketNotFound,
        BucketUpdateBucketUpdateInsufficientPermissions,
        BucketUpdateBucketUpdateNotificationConfigurationError,
        BucketUpdateBucketUpdateNotificationTopicNotFound,
        BucketUpdateBucketUpdateSnsInvalid,
    ]:
        query = gql(
            """
            mutation bucketUpdate($name: String!, $input: BucketUpdateInput!) {
              bucketUpdate(name: $name, input: $input) {
                ...BucketUpdateResultSelection
              }
            }

            fragment BucketConfigSelection on BucketConfig {
              name
              title
              iconUrl
              description
              overviewUrl
              tags
              relevanceScore
              lastIndexed
              snsNotificationArn
              scannerParallelShardsDepth
              skipMetaDataIndexing
              fileExtensionsToIndex
              indexContentBytes
              prefixes
            }

            fragment BucketUpdateResultSelection on BucketUpdateResult {
              __typename
              ... on BucketUpdateSuccess {
                bucketConfig {
                  ...BucketConfigSelection
                }
              }
              ... on InsufficientPermissions {
                message
              }
            }
            """
        )
        variables: Dict[str, object] = {"name": name, "input": input}
        response = self.execute(query=query, operation_name="bucketUpdate", variables=variables, **kwargs)
        data = self.get_data(response)
        return BucketUpdate.model_validate(data).bucket_update

    def bucket_remove(
        self, name: str, **kwargs: Any
    ) -> Union[
        BucketRemoveBucketRemoveBucketRemoveSuccess,
        BucketRemoveBucketRemoveBucketNotFound,
        BucketRemoveBucketRemoveIndexingInProgress,
    ]:
        query = gql(
            """
            mutation bucketRemove($name: String!) {
              bucketRemove(name: $name) {
                ...BucketRemoveResultSelection
              }
            }

            fragment BucketRemoveResultSelection on BucketRemoveResult {
              __typename
            }
            """
        )
        variables: Dict[str, object] = {"name": name}
        response = self.execute(query=query, operation_name="bucketRemove", variables=variables, **kwargs)
        data = self.get_data(response)
        return BucketRemove.model_validate(data).bucket_remove



# Generated by ariadne-codegen
# Source: queries.graphql

from typing import Literal, Optional, Union

from pydantic import Field

from .base_model import BaseModel
from .fragments import InvalidInputSelection, OperationErrorSelection


class UsersDelete(BaseModel):
    admin: "UsersDeleteAdmin"


class UsersDeleteAdmin(BaseModel):
    user: "UsersDeleteAdminUser"


class UsersDeleteAdminUser(BaseModel):
    mutate: Optional["UsersDeleteAdminUserMutate"]


class UsersDeleteAdminUserMutate(BaseModel):
    delete: Union[
        "UsersDeleteAdminUserMutateDeleteOk",
        "UsersDeleteAdminUserMutateDeleteInvalidInput",
        "UsersDeleteAdminUserMutateDeleteOperationError",
    ] = Field(discriminator="typename__")


class UsersDeleteAdminUserMutateDeleteOk(BaseModel):
    typename__: Literal["Ok"] = Field(alias="__typename")


class UsersDeleteAdminUserMutateDeleteInvalidInput(InvalidInputSelection):
    typename__: Literal["InvalidInput"] = Field(alias="__typename")


class UsersDeleteAdminUserMutateDeleteOperationError(OperationErrorSelection):
    typename__: Literal["OperationError"] = Field(alias="__typename")


UsersDelete.model_rebuild()
UsersDeleteAdmin.model_rebuild()
UsersDeleteAdminUser.model_rebuild()
UsersDeleteAdminUserMutate.model_rebuild()



from typing import List, Optional

from . import _graphql_client, exceptions, types, util


def get(name: str) -> Optional[types.User]:
    """
    Get a specific user from the registry. Return `None` if the user does not exist.

    Args:
        name: Username of user to get.
    """
    result = util.get_client().users_get(name=name)
    if result is None:
        return None
    return types.User(**result.model_dump())


def list() -> List[types.User]:
    """
    Get a list of all users in the registry.
    """
    return [types.User(**u.model_dump()) for u in util.get_client().users_list()]


def create(name: str, email: str, role: str, extra_roles: Optional[List[str]] = None) -> types.User:
    """
    Create a new user in the registry.

    Args:
        name: Username of user to create.
        email: Email of user to create.
        role: Active role of the user.
        extra_roles: Additional roles to assign to the user.
    """

    return util.handle_user_mutation(
        util.get_client().users_create(
            input=_graphql_client.UserInput(name=name, email=email, role=role, extraRoles=extra_roles)
        )
    )


def delete(name: str) -> None:
    """
    Delete user from the registry.

    Args:
        name: Username of user to delete.
    """
    result = util.get_client().users_delete(name=name)
    if result is None:
        raise exceptions.UserNotFoundError
    util.handle_errors(result.delete)


def set_email(name: str, email: str) -> types.User:
    """
    Set the email for a user.

    Args:
        name: Username of user to update.
        email: Email to set for the user.
    """
    result = util.get_client().users_set_email(name=name, email=email)
    if result is None:
        raise exceptions.UserNotFoundError
    return util.handle_user_mutation(result.set_email)


def set_admin(name: str, admin: bool) -> types.User:
    """
    Set the admin status for a user.

    Args:
        name: Username of user to update.
        admin: Admin status to set for the user.
    """
    result = util.get_client().users_set_admin(name=name, admin=admin)
    if result is None:
        raise exceptions.UserNotFoundError
    return util.handle_user_mutation(result.set_admin)


def set_active(name: str, active: bool) -> types.User:
    """
    Set the active status for a user.

    Args:
        name: Username of user to update.
        active: Active status to set for the user.
    """
    result = util.get_client().users_set_active(name=name, active=active)
    if result is None:
        raise exceptions.UserNotFoundError
    return util.handle_user_mutation(result.set_active)


def reset_password(name: str) -> None:
    """
    Reset the password for a user.

    Args:
        name: Username of user to update.
    """
    result = util.get_client().users_reset_password(name=name)
    if result is None:
        raise exceptions.UserNotFoundError
    util.handle_errors(result.reset_password)


def set_role(
    name: str,
    role: str,
    extra_roles: Optional[List[str]] = None,
    *,
    append: bool = False,
) -> types.User:
    """
    Set the active and extra roles for a user.

    Args:
        name: Username of user to update.
        role: Role to be set as the active role.
        extra_roles: Additional roles to assign to the user.
        append: If True, append the extra roles to the existing roles. If False, replace the existing roles.
    """
    result = util.get_client().users_set_role(name=name, role=role, extra_roles=extra_roles, append=append)
    if result is None:
        raise exceptions.UserNotFoundError
    return util.handle_user_mutation(result.set_role)


def add_roles(name: str, roles: List[str]) -> types.User:
    """
    Add roles to a user.

    Args:
        name: Username of user to update.
        roles: Roles to add to the user.
    """
    result = util.get_client().users_add_roles(name=name, roles=roles)
    if result is None:
        raise exceptions.UserNotFoundError
    return util.handle_user_mutation(result.add_roles)


def remove_roles(
    name: str,
    roles: List[str],
    fallback: Optional[str] = None,
) -> types.User:
    """
    Remove roles from a user.

    Args:
        name: Username of user to update.
        roles: Roles to remove from the user.
        fallback: If set, the role to assign to the user if the active role is removed.
    """
    result = util.get_client().users_remove_roles(name=name, roles=roles, fallback=fallback)
    if result is None:
        raise exceptions.UserNotFoundError
    return util.handle_user_mutation(result.remove_roles)



# Generated by ariadne-codegen
# Source: queries.graphql

from typing import Literal, Union

from pydantic import Field

from .base_model import BaseModel
from .fragments import BucketConfigSelection


class BucketAdd(BaseModel):
    bucket_add: Union[
        "BucketAddBucketAddBucketAddSuccess",
        "BucketAddBucketAddBucketAlreadyAdded",
        "BucketAddBucketAddBucketDoesNotExist",
        "BucketAddBucketAddBucketFileExtensionsToIndexInvalid",
        "BucketAddBucketAddBucketIndexContentBytesInvalid",
        "BucketAddBucketAddInsufficientPermissions",
        "BucketAddBucketAddNotificationConfigurationError",
        "BucketAddBucketAddNotificationTopicNotFound",
        "BucketAddBucketAddSnsInvalid",
        "BucketAddBucketAddSubscriptionInvalid",
    ] = Field(alias="bucketAdd", discriminator="typename__")


class BucketAddBucketAddBucketAddSuccess(BaseModel):
    typename__: Literal["BucketAddSuccess"] = Field(alias="__typename")
    bucket_config: "BucketAddBucketAddBucketAddSuccessBucketConfig" = Field(alias="bucketConfig")


class BucketAddBucketAddBucketAddSuccessBucketConfig(BucketConfigSelection):
    pass


class BucketAddBucketAddBucketAlreadyAdded(BaseModel):
    typename__: Literal["BucketAlreadyAdded"] = Field(alias="__typename")


class BucketAddBucketAddBucketDoesNotExist(BaseModel):
    typename__: Literal["BucketDoesNotExist"] = Field(alias="__typename")


class BucketAddBucketAddBucketFileExtensionsToIndexInvalid(BaseModel):
    typename__: Literal["BucketFileExtensionsToIndexInvalid"] = Field(alias="__typename")


class BucketAddBucketAddBucketIndexContentBytesInvalid(BaseModel):
    typename__: Literal["BucketIndexContentBytesInvalid"] = Field(alias="__typename")


class BucketAddBucketAddInsufficientPermissions(BaseModel):
    typename__: Literal["InsufficientPermissions"] = Field(alias="__typename")
    message: str


class BucketAddBucketAddNotificationConfigurationError(BaseModel):
    typename__: Literal["NotificationConfigurationError"] = Field(alias="__typename")


class BucketAddBucketAddNotificationTopicNotFound(BaseModel):
    typename__: Literal["NotificationTopicNotFound"] = Field(alias="__typename")


class BucketAddBucketAddSnsInvalid(BaseModel):
    typename__: Literal["SnsInvalid"] = Field(alias="__typename")


class BucketAddBucketAddSubscriptionInvalid(BaseModel):
    typename__: Literal["SubscriptionInvalid"] = Field(alias="__typename")


BucketAdd.model_rebuild()
BucketAddBucketAddBucketAddSuccess.model_rebuild()



# Generated by ariadne-codegen
# Source: queries.graphql

from typing import Literal, Optional, Union

from pydantic import Field

from .base_model import BaseModel
from .fragments import InvalidInputSelection, OperationErrorSelection, UserSelection


class UsersAddRoles(BaseModel):
    admin: "UsersAddRolesAdmin"


class UsersAddRolesAdmin(BaseModel):
    user: "UsersAddRolesAdminUser"


class UsersAddRolesAdminUser(BaseModel):
    mutate: Optional["UsersAddRolesAdminUserMutate"]


class UsersAddRolesAdminUserMutate(BaseModel):
    add_roles: Union[
        "UsersAddRolesAdminUserMutateAddRolesUser",
        "UsersAddRolesAdminUserMutateAddRolesInvalidInput",
        "UsersAddRolesAdminUserMutateAddRolesOperationError",
    ] = Field(alias="addRoles", discriminator="typename__")


class UsersAddRolesAdminUserMutateAddRolesUser(UserSelection):
    typename__: Literal["User"] = Field(alias="__typename")


class UsersAddRolesAdminUserMutateAddRolesInvalidInput(InvalidInputSelection):
    typename__: Literal["InvalidInput"] = Field(alias="__typename")


class UsersAddRolesAdminUserMutateAddRolesOperationError(OperationErrorSelection):
    typename__: Literal["OperationError"] = Field(alias="__typename")


UsersAddRoles.model_rebuild()
UsersAddRolesAdmin.model_rebuild()
UsersAddRolesAdminUser.model_rebuild()
UsersAddRolesAdminUserMutate.model_rebuild()



# Generated by ariadne-codegen
# Source: queries.graphql

from typing import Literal, Optional, Union

from pydantic import Field

from .base_model import BaseModel
from .fragments import InvalidInputSelection, OperationErrorSelection, UserSelection


class UsersSetAdmin(BaseModel):
    admin: "UsersSetAdminAdmin"


class UsersSetAdminAdmin(BaseModel):
    user: "UsersSetAdminAdminUser"


class UsersSetAdminAdminUser(BaseModel):
    mutate: Optional["UsersSetAdminAdminUserMutate"]


class UsersSetAdminAdminUserMutate(BaseModel):
    set_admin: Union[
        "UsersSetAdminAdminUserMutateSetAdminUser",
        "UsersSetAdminAdminUserMutateSetAdminInvalidInput",
        "UsersSetAdminAdminUserMutateSetAdminOperationError",
    ] = Field(alias="setAdmin", discriminator="typename__")


class UsersSetAdminAdminUserMutateSetAdminUser(UserSelection):
    typename__: Literal["User"] = Field(alias="__typename")


class UsersSetAdminAdminUserMutateSetAdminInvalidInput(InvalidInputSelection):
    typename__: Literal["InvalidInput"] = Field(alias="__typename")


class UsersSetAdminAdminUserMutateSetAdminOperationError(OperationErrorSelection):
    typename__: Literal["OperationError"] = Field(alias="__typename")


UsersSetAdmin.model_rebuild()
UsersSetAdminAdmin.model_rebuild()
UsersSetAdminAdminUser.model_rebuild()
UsersSetAdminAdminUserMutate.model_rebuild()



# Generated by ariadne-codegen
# Source: queries.graphql

from typing import List

from .base_model import BaseModel
from .fragments import UserSelection


class UsersList(BaseModel):
    admin: "UsersListAdmin"


class UsersListAdmin(BaseModel):
    user: "UsersListAdminUser"


class UsersListAdminUser(BaseModel):
    list: List["UsersListAdminUserList"]


class UsersListAdminUserList(UserSelection):
    pass


UsersList.model_rebuild()
UsersListAdmin.model_rebuild()
UsersListAdminUser.model_rebuild()



# Generated by ariadne-codegen
# Source: queries.graphql

from typing import Literal, Optional, Union

from pydantic import Field

from .base_model import BaseModel
from .fragments import InvalidInputSelection, OperationErrorSelection


class UsersResetPassword(BaseModel):
    admin: "UsersResetPasswordAdmin"


class UsersResetPasswordAdmin(BaseModel):
    user: "UsersResetPasswordAdminUser"


class UsersResetPasswordAdminUser(BaseModel):
    mutate: Optional["UsersResetPasswordAdminUserMutate"]


class UsersResetPasswordAdminUserMutate(BaseModel):
    reset_password: Union[
        "UsersResetPasswordAdminUserMutateResetPasswordOk",
        "UsersResetPasswordAdminUserMutateResetPasswordInvalidInput",
        "UsersResetPasswordAdminUserMutateResetPasswordOperationError",
    ] = Field(alias="resetPassword", discriminator="typename__")


class UsersResetPasswordAdminUserMutateResetPasswordOk(BaseModel):
    typename__: Literal["Ok"] = Field(alias="__typename")


class UsersResetPasswordAdminUserMutateResetPasswordInvalidInput(InvalidInputSelection):
    typename__: Literal["InvalidInput"] = Field(alias="__typename")


class UsersResetPasswordAdminUserMutateResetPasswordOperationError(OperationErrorSelection):
    typename__: Literal["OperationError"] = Field(alias="__typename")


UsersResetPassword.model_rebuild()
UsersResetPasswordAdmin.model_rebuild()
UsersResetPasswordAdminUser.model_rebuild()
UsersResetPasswordAdminUserMutate.model_rebuild()



# Generated by ariadne-codegen
# Source: queries.graphql

from typing import Annotated, List, Literal, Union

from pydantic import Field

from .base_model import BaseModel
from .fragments import ManagedRoleSelection, UnmanagedRoleSelection


class RolesList(BaseModel):
    roles: List[
        Annotated[
            Union["RolesListRolesUnmanagedRole", "RolesListRolesManagedRole"],
            Field(discriminator="typename__"),
        ]
    ]


class RolesListRolesUnmanagedRole(UnmanagedRoleSelection):
    typename__: Literal["UnmanagedRole"] = Field(alias="__typename")


class RolesListRolesManagedRole(ManagedRoleSelection):
    typename__: Literal["ManagedRole"] = Field(alias="__typename")


RolesList.model_rebuild()



import typing as T

from . import types, util


def get() -> T.Optional[types.SSOConfig]:
    """
    Get the current SSO configuration.
    """
    result = util.get_client().sso_config_get()
    return None if result is None else types.SSOConfig(**result.model_dump())


def set(config: T.Optional[str]) -> T.Optional[types.SSOConfig]:
    """
    Set the SSO configuration. Pass `None` to remove SSO configuration.
    """
    result = util.get_client().sso_config_set(config)
    return None if result is None else types.SSOConfig(**util.handle_errors(result).model_dump())



# Generated by ariadne-codegen
# Source: queries.graphql

from typing import Literal, Optional, Union

from pydantic import Field

from .base_model import BaseModel
from .fragments import InvalidInputSelection, OperationErrorSelection, UserSelection


class UsersSetActive(BaseModel):
    admin: "UsersSetActiveAdmin"


class UsersSetActiveAdmin(BaseModel):
    user: "UsersSetActiveAdminUser"


class UsersSetActiveAdminUser(BaseModel):
    mutate: Optional["UsersSetActiveAdminUserMutate"]


class UsersSetActiveAdminUserMutate(BaseModel):
    set_active: Union[
        "UsersSetActiveAdminUserMutateSetActiveUser",
        "UsersSetActiveAdminUserMutateSetActiveInvalidInput",
        "UsersSetActiveAdminUserMutateSetActiveOperationError",
    ] = Field(alias="setActive", discriminator="typename__")


class UsersSetActiveAdminUserMutateSetActiveUser(UserSelection):
    typename__: Literal["User"] = Field(alias="__typename")


class UsersSetActiveAdminUserMutateSetActiveInvalidInput(InvalidInputSelection):
    typename__: Literal["InvalidInput"] = Field(alias="__typename")


class UsersSetActiveAdminUserMutateSetActiveOperationError(OperationErrorSelection):
    typename__: Literal["OperationError"] = Field(alias="__typename")


UsersSetActive.model_rebuild()
UsersSetActiveAdmin.model_rebuild()
UsersSetActiveAdminUser.model_rebuild()
UsersSetActiveAdminUserMutate.model_rebuild()



# Generated by ariadne-codegen

# This is
# https://github.com/mirumee/ariadne-codegen/blob/5bfd63c5e7e3a8cc5293eb94deee638b7adab98d/ariadne_codegen/client_generators/dependencies/base_client.py
# modified to use our requests session instead of httpx.
# pylint: disable=relative-beyond-top-level
# fmt: off
import json
from typing import IO, Any, Dict, List, Optional, Tuple, TypeVar, cast

import requests
from pydantic import BaseModel
from pydantic_core import to_jsonable_python

from quilt3 import session

from .base_model import UNSET, Upload
from .exceptions import (
    GraphQLClientGraphQLMultiError,
    GraphQLClientHttpError,
    GraphQLClientInvalidResponseError,
)

Self = TypeVar("Self", bound="BaseClient")


class BaseClient:
    def __init__(
        self,
    ) -> None:
        self.url = session.get_registry_url() + "/graphql"

        self.http_client = session.get_session()

    def __enter__(self: Self) -> Self:
        return self

    def __exit__(
        self,
        exc_type: object,
        exc_val: object,
        exc_tb: object,
    ) -> None:
        self.http_client.close()

    def execute(
        self,
        query: str,
        operation_name: Optional[str] = None,
        variables: Optional[Dict[str, Any]] = None,
        **kwargs: Any,
    ) -> requests.Response:
        processed_variables, files, files_map = self._process_variables(variables)

        if files and files_map:
            return self._execute_multipart(
                query=query,
                operation_name=operation_name,
                variables=processed_variables,
                files=files,
                files_map=files_map,
                **kwargs,
            )

        return self._execute_json(
            query=query,
            operation_name=operation_name,
            variables=processed_variables,
            **kwargs,
        )

    def get_data(self, response: requests.Response) -> Dict[str, Any]:
        if not 200 <= response.status_code < 300:
            raise GraphQLClientHttpError(
                status_code=response.status_code, response=response
            )

        try:
            response_json = response.json()
        except ValueError as exc:
            raise GraphQLClientInvalidResponseError(response=response) from exc

        if (not isinstance(response_json, dict)) or (
            "data" not in response_json and "errors" not in response_json
        ):
            raise GraphQLClientInvalidResponseError(response=response)

        data = response_json.get("data")
        errors = response_json.get("errors")

        if errors:
            raise GraphQLClientGraphQLMultiError.from_errors_dicts(
                errors_dicts=errors, data=data
            )

        return cast(Dict[str, Any], data)

    def _process_variables(
        self, variables: Optional[Dict[str, Any]]
    ) -> Tuple[
        Dict[str, Any], Dict[str, Tuple[str, IO[bytes], str]], Dict[str, List[str]]
    ]:
        if not variables:
            return {}, {}, {}

        serializable_variables = self._convert_dict_to_json_serializable(variables)
        return self._get_files_from_variables(serializable_variables)

    def _convert_dict_to_json_serializable(
        self, dict_: Dict[str, Any]
    ) -> Dict[str, Any]:
        return {
            key: self._convert_value(value)
            for key, value in dict_.items()
            if value is not UNSET
        }

    def _convert_value(self, value: Any) -> Any:
        if isinstance(value, BaseModel):
            return value.model_dump(by_alias=True, exclude_unset=True)
        if isinstance(value, list):
            return [self._convert_value(item) for item in value]
        return value

    def _get_files_from_variables(
        self, variables: Dict[str, Any]
    ) -> Tuple[
        Dict[str, Any], Dict[str, Tuple[str, IO[bytes], str]], Dict[str, List[str]]
    ]:
        files_map: Dict[str, List[str]] = {}
        files_list: List[Upload] = []

        def separate_files(path: str, obj: Any) -> Any:
            if isinstance(obj, list):
                nulled_list = []
                for index, value in enumerate(obj):
                    value = separate_files(f"{path}.{index}", value)
                    nulled_list.append(value)
                return nulled_list

            if isinstance(obj, dict):
                nulled_dict = {}
                for key, value in obj.items():
                    value = separate_files(f"{path}.{key}", value)
                    nulled_dict[key] = value
                return nulled_dict

            if isinstance(obj, Upload):
                if obj in files_list:
                    file_index = files_list.index(obj)
                    files_map[str(file_index)].append(path)
                else:
                    file_index = len(files_list)
                    files_list.append(obj)
                    files_map[str(file_index)] = [path]
                return None

            return obj

        nulled_variables = separate_files("variables", variables)
        files: Dict[str, Tuple[str, IO[bytes], str]] = {
            str(i): (file_.filename, cast(IO[bytes], file_.content), file_.content_type)
            for i, file_ in enumerate(files_list)
        }
        return nulled_variables, files, files_map

    def _execute_multipart(
        self,
        query: str,
        operation_name: Optional[str],
        variables: Dict[str, Any],
        files: Dict[str, Tuple[str, IO[bytes], str]],
        files_map: Dict[str, List[str]],
        **kwargs: Any,
    ) -> requests.Response:
        data = {
            "operations": json.dumps(
                {
                    "query": query,
                    "operationName": operation_name,
                    "variables": variables,
                },
                default=to_jsonable_python,
            ),
            "map": json.dumps(files_map, default=to_jsonable_python),
        }

        return self.http_client.post(url=self.url, data=data, files=files, **kwargs)

    def _execute_json(
        self,
        query: str,
        operation_name: Optional[str],
        variables: Dict[str, Any],
        **kwargs: Any,
    ) -> requests.Response:
        headers: Dict[str, str] = {"Content-Type": "application/json"}
        headers.update(kwargs.get("headers", {}))

        merged_kwargs: Dict[str, Any] = kwargs.copy()
        merged_kwargs["headers"] = headers

        return self.http_client.post(
            url=self.url,
            data=json.dumps(
                {
                    "query": query,
                    "operationName": operation_name,
                    "variables": variables,
                },
                default=to_jsonable_python,
            ),
            **merged_kwargs,
        )



# Generated by ariadne-codegen
# Source: queries.graphql

from typing import Literal, Optional, Union

from pydantic import Field

from .base_model import BaseModel
from .fragments import InvalidInputSelection, OperationErrorSelection, UserSelection


class UsersSetEmail(BaseModel):
    admin: "UsersSetEmailAdmin"


class UsersSetEmailAdmin(BaseModel):
    user: "UsersSetEmailAdminUser"


class UsersSetEmailAdminUser(BaseModel):
    mutate: Optional["UsersSetEmailAdminUserMutate"]


class UsersSetEmailAdminUserMutate(BaseModel):
    set_email: Union[
        "UsersSetEmailAdminUserMutateSetEmailUser",
        "UsersSetEmailAdminUserMutateSetEmailInvalidInput",
        "UsersSetEmailAdminUserMutateSetEmailOperationError",
    ] = Field(alias="setEmail", discriminator="typename__")


class UsersSetEmailAdminUserMutateSetEmailUser(UserSelection):
    typename__: Literal["User"] = Field(alias="__typename")


class UsersSetEmailAdminUserMutateSetEmailInvalidInput(InvalidInputSelection):
    typename__: Literal["InvalidInput"] = Field(alias="__typename")


class UsersSetEmailAdminUserMutateSetEmailOperationError(OperationErrorSelection):
    typename__: Literal["OperationError"] = Field(alias="__typename")


UsersSetEmail.model_rebuild()
UsersSetEmailAdmin.model_rebuild()
UsersSetEmailAdminUser.model_rebuild()
UsersSetEmailAdminUserMutate.model_rebuild()
